# -*- coding: utf-8 -*-
#
# Copyright (c) 2015, Alcatel-Lucent Inc, 2017 Nokia
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the copyright holder nor the names of its contributors
#       may be used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import logging
import importlib
import re
import pkg_resources

from bambou.exceptions import BambouHTTPError
from .printer import Printer


class Utils(object):
    """ Utils """

    # TODO: remove that custom thing
    INVARIANT_RESOURCES = ["qos", "vrs", "cms", "statistics", "licensestatus", "vrsmetrics", "ltestatistics", "bulkstatistics"]

    @classmethod
    def _clean_name(cls, string):
        """ String cleaning for specific cases

            This is very specific and is used to force
            some underscore while using get_python_name.

            Args:
                string: the string to clean

            Returns:
                Returns a clean string
        """
        rep = {
            "VPort": "Vport",
            "IPID": "IpID"
        }

        rep = dict((re.escape(k), v) for k, v in rep.iteritems())
        pattern = re.compile("|".join(rep.keys()))
        return pattern.sub(lambda m: rep[re.escape(m.group(0))], string)

    @classmethod
    def get_python_name(cls, name):
        """ Transform a given name to python name """
        first_cap_re = re.compile("(.)([A-Z](?!s([A-Z])*)[a-z]+)")
        all_cap_re = re.compile("([a-z0-9])([A-Z])")

        s1 = first_cap_re.sub(r"\1_\2", Utils._clean_name(name))
        return all_cap_re.sub(r"\1_\2", s1).lower()

    @classmethod
    def get_singular_name(cls, entity_name_plural):
        """ Returns the singular name of the plural name """

        if entity_name_plural in Utils.INVARIANT_RESOURCES:
            return entity_name_plural

        if entity_name_plural[-3:] == "ies":
            return entity_name_plural[:-3] + "y"

        if entity_name_plural[-1] == "s":
            return entity_name_plural[:-1]

        return entity_name_plural

    @classmethod
    def get_entity_name_plural(cls, singular_name):
        """ Returns the plural name of the singular name """

        if singular_name in Utils.INVARIANT_RESOURCES:
            return singular_name

        vowels = ["a", "e", "i", "o", "u", "y"]
        if singular_name[-1:] == "y" and singular_name[-2] not in vowels:
            return singular_name[:-1] + "ies"

        if singular_name[-1:] == "s":
            return singular_name

        return singular_name + "s"

    @classmethod
    def get_vspk_version(cls, version):
        """ Get the vspk version according to the given version

            Args:
                version (int): the version

            Returns:
                version as string

            Example:
                get_vspk_version(3.1)
                >>> v3_1

        """
        return ("v%s" % version).replace(".", "_")


class SDKInspector(object):
    """ Utils to access SDK objects

    """

    def __init__(self, version=None):
        """ Initializes

        """
        if version:
            self._version = Utils.get_vspk_version(version)

        self._objects_mapping = {}
        self._ignored_resources = []
        self._sdk = None
        self._logger = None

        self._load_objects()

    def _load_objects(self):
        """ Load objects in a temporary database

        """
        self._get_package()

        object_names = [name for name in dir(self._sdk) if name != "NUVSDSession" and name != "SDKInfo" and name.startswith("NU") and not name.endswith("Fetcher")]

        for object_name in object_names:
            obj = getattr(self._sdk, object_name)
            self._objects_mapping[obj.rest_name] = object_name

    def _get_package(self):
        """ Returns sdk package

        """
        if self._sdk is None:
            try:
                self._sdk = importlib.import_module("vspk.%s" % self._version)
                self._logger = importlib.import_module("vspk.utils").set_log_level
                self._ignored_resources = [self._sdk.SDKInfo.root_object_class().rest_name]

                # Printer.info("Imported vspk.%s." % self._version)
            except ImportError:
                self._sdk = importlib.import_module("vspk")
            except ImportError as error:
                Printer.raise_error("Please install requirements using command line 'pip install -r requirements.txt'.\n%s" % error)

        return self._sdk

    def get_all_objects(self):
        """ Returns all objects available

        """
        resources = self._objects_mapping.keys()
        resources = [Utils.get_entity_name_plural(name) for name in resources if name not in self._ignored_resources]

        return resources

    def get_class(self, name):
        """ Get a SDK class object
            Args:
                name: the name of the object
            Returns:
                a SDK class object
        """
        if name in self._objects_mapping:
            classname = self._objects_mapping[name]

            klass = None
            try:
                klass = getattr(self._sdk, classname)
            except:
                Printer.raise_error('Unknown class %s' % classname)

            return klass

        Printer.raise_error('Unknown object named %s' % name)

    def get_instance(self, name):
        """ Get SDK object instance according to a given name

            Args:
                name: the name of the object

            Returns:
                A SDK object or raise an exception
        """
        klass = self.get_class(name)
        return klass()

    def get_parent(self, parent_infos, root_object):
        """ Get SDK parent object if possible
            Otherwise it will take the user

            Args:
                parent_infos: a list composed of (parent_name, uuid)

            Returns:
                A parent if possible otherwise the user in session

        """
        if parent_infos and len(parent_infos) == 2:
            name = parent_infos[0]
            uuid = parent_infos[1]

            singular_name = Utils.get_singular_name(name)
            parent = self.get_instance(singular_name)
            parent.id = uuid

            try:
                (parent, connection) = parent.fetch()
            except Exception, ex:
                Printer.raise_error("Failed fetching parent %s with uuid %s\n%s" % (name, uuid, ex))

            return parent

        return root_object

    def get_user_session(self, args):
        """ Get api key

            Args:
                username: username to get an api key
                password: password to get an api key
                api: URL of the API endpoint
                enterprise: Name of the enterprise to connect

            Returns:
                Returns an API Key if everything works fine
        """
        self._set_verbose_mode(args.verbose)
        session = self._sdk.NUVSDSession(username=args.username, password=args.password, enterprise=args.enterprise, api_url=args.api)
        try:
            session.start()
        except BambouHTTPError as error:
            status_code = error.connection.response.status_code
            if status_code == 401:
                Printer.raise_error("Could not log on %s (API %s) with username=%s password=%s enterprise=%s" % (args.api, args.version, args.username, args.password, args.enterprise))
            else:
                Printer.raise_error("Cannot access %s [HTTP %s]. Current vspk version tried to connect to the Server API %s" % (args.api, status_code, args.version))

        root_object = session.root_object

        if root_object.api_key is None:
            Printer.raise_error("Could not get a valid API key. Activate verbose mode for more information")

        return session

    def _set_verbose_mode(self, verbose):
        """ Defines verbosity

            Args:
                verbose: Boolean to activate or deactivate DEBUG mode

        """
        pass
        if verbose:
            Printer.info("Verbose mode is now activated.")
            self._logger(logging.DEBUG)
        else:
            self._logger(logging.ERROR)


# -*- coding: utf-8 -*-
#
# Copyright (c) 2015, Alcatel-Lucent Inc, 2017 Nokia
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the copyright holder nor the names of its contributors
#       may be used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.




from .fetchers import NUTiersFetcher


from .fetchers import NUApplicationperformancemanagementsFetcher

from bambou import NURESTObject


class NUPerformanceMonitor(NURESTObject):
    """ Represents a PerformanceMonitor in the VSD

        Notes:
            To enable the network performance monitoring between NSGs in an NSG Group and NSG-UBRs in an NSG-UBR Group. 
    """

    __rest_name__ = "performancemonitor"
    __resource_name__ = "performancemonitors"

    
    ## Constants
    
    CONST_SERVICE_CLASS_H = "H"
    
    CONST_PROBE_TYPE_IPSEC_AND_VXLAN = "IPSEC_AND_VXLAN"
    
    CONST_ENTITY_SCOPE_GLOBAL = "GLOBAL"
    
    CONST_SERVICE_CLASS_A = "A"
    
    CONST_SERVICE_CLASS_B = "B"
    
    CONST_SERVICE_CLASS_C = "C"
    
    CONST_SERVICE_CLASS_D = "D"
    
    CONST_SERVICE_CLASS_E = "E"
    
    CONST_SERVICE_CLASS_F = "F"
    
    CONST_SERVICE_CLASS_G = "G"
    
    CONST_PROBE_TYPE_HTTP = "HTTP"
    
    CONST_PROBE_TYPE_ONEWAY = "ONEWAY"
    
    CONST_ENTITY_SCOPE_ENTERPRISE = "ENTERPRISE"
    
    

    def __init__(self, **kwargs):
        """ Initializes a PerformanceMonitor instance

            Notes:
                You can specify all parameters while calling this methods.
                A special argument named `data` will enable you to load the
                object from a Python dictionary

            Examples:
                >>> performancemonitor = NUPerformanceMonitor(id=u'xxxx-xxx-xxx-xxx', name=u'PerformanceMonitor')
                >>> performancemonitor = NUPerformanceMonitor(data=my_dict)
        """

        super(NUPerformanceMonitor, self).__init__()

        # Read/Write Attributes
        
        self._name = None
        self._last_updated_by = None
        self._payload_size = None
        self._read_only = None
        self._service_class = None
        self._description = None
        self._interval = None
        self._entity_scope = None
        self._hold_down_timer = None
        self._probe_type = None
        self._number_of_packets = None
        self._external_id = None
        
        self.expose_attribute(local_name="name", remote_name="name", attribute_type=str, is_required=True, is_unique=False)
        self.expose_attribute(local_name="last_updated_by", remote_name="lastUpdatedBy", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="payload_size", remote_name="payloadSize", attribute_type=int, is_required=False, is_unique=False)
        self.expose_attribute(local_name="read_only", remote_name="readOnly", attribute_type=bool, is_required=False, is_unique=False)
        self.expose_attribute(local_name="service_class", remote_name="serviceClass", attribute_type=str, is_required=False, is_unique=False, choices=[u'A', u'B', u'C', u'D', u'E', u'F', u'G', u'H'])
        self.expose_attribute(local_name="description", remote_name="description", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="interval", remote_name="interval", attribute_type=int, is_required=True, is_unique=False)
        self.expose_attribute(local_name="entity_scope", remote_name="entityScope", attribute_type=str, is_required=False, is_unique=False, choices=[u'ENTERPRISE', u'GLOBAL'])
        self.expose_attribute(local_name="hold_down_timer", remote_name="holdDownTimer", attribute_type=int, is_required=False, is_unique=False)
        self.expose_attribute(local_name="probe_type", remote_name="probeType", attribute_type=str, is_required=False, is_unique=False, choices=[u'HTTP', u'IPSEC_AND_VXLAN', u'ONEWAY'])
        self.expose_attribute(local_name="number_of_packets", remote_name="numberOfPackets", attribute_type=int, is_required=True, is_unique=False)
        self.expose_attribute(local_name="external_id", remote_name="externalID", attribute_type=str, is_required=False, is_unique=True)
        

        # Fetchers
        
        
        self.tiers = NUTiersFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.applicationperformancemanagements = NUApplicationperformancemanagementsFetcher.fetcher_with_object(parent_object=self, relationship="member")
        

        self._compute_args(**kwargs)

    # Properties
    
    @property
    def name(self):
        """ Get name value.

            Notes:
                Name of the application group probe

                
        """
        return self._name

    @name.setter
    def name(self, value):
        """ Set name value.

            Notes:
                Name of the application group probe

                
        """
        self._name = value

    
    @property
    def last_updated_by(self):
        """ Get last_updated_by value.

            Notes:
                ID of the user who last updated the object.

                
                This attribute is named `lastUpdatedBy` in VSD API.
                
        """
        return self._last_updated_by

    @last_updated_by.setter
    def last_updated_by(self, value):
        """ Set last_updated_by value.

            Notes:
                ID of the user who last updated the object.

                
                This attribute is named `lastUpdatedBy` in VSD API.
                
        """
        self._last_updated_by = value

    
    @property
    def payload_size(self):
        """ Get payload_size value.

            Notes:
                Payload size (This is a mandatory field if the networkProbeType = ONEWAY, and optional for probeType = HTTP,IPSEC_AND_VXLAN)

                
                This attribute is named `payloadSize` in VSD API.
                
        """
        return self._payload_size

    @payload_size.setter
    def payload_size(self, value):
        """ Set payload_size value.

            Notes:
                Payload size (This is a mandatory field if the networkProbeType = ONEWAY, and optional for probeType = HTTP,IPSEC_AND_VXLAN)

                
                This attribute is named `payloadSize` in VSD API.
                
        """
        self._payload_size = value

    
    @property
    def read_only(self):
        """ Get read_only value.

            Notes:
                Determines whether this entity is read only.  Read only objects cannot be modified or deleted.

                
                This attribute is named `readOnly` in VSD API.
                
        """
        return self._read_only

    @read_only.setter
    def read_only(self, value):
        """ Set read_only value.

            Notes:
                Determines whether this entity is read only.  Read only objects cannot be modified or deleted.

                
                This attribute is named `readOnly` in VSD API.
                
        """
        self._read_only = value

    
    @property
    def service_class(self):
        """ Get service_class value.

            Notes:
                Class of service to be used.  Service classes in order of priority are A, B, C, D, E, F, G, and H.

                
                This attribute is named `serviceClass` in VSD API.
                
        """
        return self._service_class

    @service_class.setter
    def service_class(self, value):
        """ Set service_class value.

            Notes:
                Class of service to be used.  Service classes in order of priority are A, B, C, D, E, F, G, and H.

                
                This attribute is named `serviceClass` in VSD API.
                
        """
        self._service_class = value

    
    @property
    def description(self):
        """ Get description value.

            Notes:
                Description of application group probe

                
        """
        return self._description

    @description.setter
    def description(self, value):
        """ Set description value.

            Notes:
                Description of application group probe

                
        """
        self._description = value

    
    @property
    def interval(self):
        """ Get interval value.

            Notes:
                interval in seconds

                
        """
        return self._interval

    @interval.setter
    def interval(self, value):
        """ Set interval value.

            Notes:
                interval in seconds

                
        """
        self._interval = value

    
    @property
    def entity_scope(self):
        """ Get entity_scope value.

            Notes:
                Specify if scope of entity is Data center or Enterprise level

                
                This attribute is named `entityScope` in VSD API.
                
        """
        return self._entity_scope

    @entity_scope.setter
    def entity_scope(self, value):
        """ Set entity_scope value.

            Notes:
                Specify if scope of entity is Data center or Enterprise level

                
                This attribute is named `entityScope` in VSD API.
                
        """
        self._entity_scope = value

    
    @property
    def hold_down_timer(self):
        """ Get hold_down_timer value.

            Notes:
                Probe Timeout in milliseconds

                
                This attribute is named `holdDownTimer` in VSD API.
                
        """
        return self._hold_down_timer

    @hold_down_timer.setter
    def hold_down_timer(self, value):
        """ Set hold_down_timer value.

            Notes:
                Probe Timeout in milliseconds

                
                This attribute is named `holdDownTimer` in VSD API.
                
        """
        self._hold_down_timer = value

    
    @property
    def probe_type(self):
        """ Get probe_type value.

            Notes:
                Type to be assigned to this probe

                
                This attribute is named `probeType` in VSD API.
                
        """
        return self._probe_type

    @probe_type.setter
    def probe_type(self, value):
        """ Set probe_type value.

            Notes:
                Type to be assigned to this probe

                
                This attribute is named `probeType` in VSD API.
                
        """
        self._probe_type = value

    
    @property
    def number_of_packets(self):
        """ Get number_of_packets value.

            Notes:
                number of packets

                
                This attribute is named `numberOfPackets` in VSD API.
                
        """
        return self._number_of_packets

    @number_of_packets.setter
    def number_of_packets(self, value):
        """ Set number_of_packets value.

            Notes:
                number of packets

                
                This attribute is named `numberOfPackets` in VSD API.
                
        """
        self._number_of_packets = value

    
    @property
    def external_id(self):
        """ Get external_id value.

            Notes:
                External object ID. Used for integration with third party systems

                
                This attribute is named `externalID` in VSD API.
                
        """
        return self._external_id

    @external_id.setter
    def external_id(self, value):
        """ Set external_id value.

            Notes:
                External object ID. Used for integration with third party systems

                
                This attribute is named `externalID` in VSD API.
                
        """
        self._external_id = value

    

    


# -*- coding: utf-8 -*-
#
# Copyright (c) 2015, Alcatel-Lucent Inc, 2017 Nokia
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the copyright holder nor the names of its contributors
#       may be used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.




from .fetchers import NUDeploymentFailuresFetcher


from .fetchers import NUMetadatasFetcher


from .fetchers import NUGlobalMetadatasFetcher


from .fetchers import NUVPortsFetcher

from bambou import NURESTObject


class NUEgressProfile(NURESTObject):
    """ Represents a EgressProfile in the VSD

        Notes:
            An Egress Profile represents an aggregation of IP, MAC and egress QoS profiles that are applied on a VPort instance.
    """

    __rest_name__ = "egressprofile"
    __resource_name__ = "egressprofiles"

    
    ## Constants
    
    CONST_ENTITY_SCOPE_GLOBAL = "GLOBAL"
    
    CONST_ENTITY_SCOPE_ENTERPRISE = "ENTERPRISE"
    
    

    def __init__(self, **kwargs):
        """ Initializes a EgressProfile instance

            Notes:
                You can specify all parameters while calling this methods.
                A special argument named `data` will enable you to load the
                object from a Python dictionary

            Examples:
                >>> egressprofile = NUEgressProfile(id=u'xxxx-xxx-xxx-xxx', name=u'EgressProfile')
                >>> egressprofile = NUEgressProfile(data=my_dict)
        """

        super(NUEgressProfile, self).__init__()

        # Read/Write Attributes
        
        self._name = None
        self._last_updated_by = None
        self._description = None
        self._entity_scope = None
        self._assoc_entity_type = None
        self._associated_ip_filter_profile_id = None
        self._associated_ip_filter_profile_name = None
        self._associated_ipv6_filter_profile_id = None
        self._associated_ipv6_filter_profile_name = None
        self._associated_mac_filter_profile_id = None
        self._associated_mac_filter_profile_name = None
        self._associated_sap_egress_qo_s_profile_id = None
        self._associated_sap_egress_qo_s_profile_name = None
        self._external_id = None
        
        self.expose_attribute(local_name="name", remote_name="name", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="last_updated_by", remote_name="lastUpdatedBy", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="description", remote_name="description", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="entity_scope", remote_name="entityScope", attribute_type=str, is_required=False, is_unique=False, choices=[u'ENTERPRISE', u'GLOBAL'])
        self.expose_attribute(local_name="assoc_entity_type", remote_name="assocEntityType", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="associated_ip_filter_profile_id", remote_name="associatedIPFilterProfileID", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="associated_ip_filter_profile_name", remote_name="associatedIPFilterProfileName", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="associated_ipv6_filter_profile_id", remote_name="associatedIPv6FilterProfileID", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="associated_ipv6_filter_profile_name", remote_name="associatedIPv6FilterProfileName", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="associated_mac_filter_profile_id", remote_name="associatedMACFilterProfileID", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="associated_mac_filter_profile_name", remote_name="associatedMACFilterProfileName", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="associated_sap_egress_qo_s_profile_id", remote_name="associatedSAPEgressQoSProfileID", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="associated_sap_egress_qo_s_profile_name", remote_name="associatedSAPEgressQoSProfileName", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="external_id", remote_name="externalID", attribute_type=str, is_required=False, is_unique=True)
        

        # Fetchers
        
        
        self.deployment_failures = NUDeploymentFailuresFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.metadatas = NUMetadatasFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.global_metadatas = NUGlobalMetadatasFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.vports = NUVPortsFetcher.fetcher_with_object(parent_object=self, relationship="child")
        

        self._compute_args(**kwargs)

    # Properties
    
    @property
    def name(self):
        """ Get name value.

            Notes:
                A customer friendly name for the Egress Profile entity.

                
        """
        return self._name

    @name.setter
    def name(self, value):
        """ Set name value.

            Notes:
                A customer friendly name for the Egress Profile entity.

                
        """
        self._name = value

    
    @property
    def last_updated_by(self):
        """ Get last_updated_by value.

            Notes:
                ID of the user who last updated the object.

                
                This attribute is named `lastUpdatedBy` in VSD API.
                
        """
        return self._last_updated_by

    @last_updated_by.setter
    def last_updated_by(self, value):
        """ Set last_updated_by value.

            Notes:
                ID of the user who last updated the object.

                
                This attribute is named `lastUpdatedBy` in VSD API.
                
        """
        self._last_updated_by = value

    
    @property
    def description(self):
        """ Get description value.

            Notes:
                A customer friendly description of the Egress Profile entity.

                
        """
        return self._description

    @description.setter
    def description(self, value):
        """ Set description value.

            Notes:
                A customer friendly description of the Egress Profile entity.

                
        """
        self._description = value

    
    @property
    def entity_scope(self):
        """ Get entity_scope value.

            Notes:
                Specify if scope of entity is Data center or Enterprise level

                
                This attribute is named `entityScope` in VSD API.
                
        """
        return self._entity_scope

    @entity_scope.setter
    def entity_scope(self, value):
        """ Set entity_scope value.

            Notes:
                Specify if scope of entity is Data center or Enterprise level

                
                This attribute is named `entityScope` in VSD API.
                
        """
        self._entity_scope = value

    
    @property
    def assoc_entity_type(self):
        """ Get assoc_entity_type value.

            Notes:
                Type of parent entity

                
                This attribute is named `assocEntityType` in VSD API.
                
        """
        return self._assoc_entity_type

    @assoc_entity_type.setter
    def assoc_entity_type(self, value):
        """ Set assoc_entity_type value.

            Notes:
                Type of parent entity

                
                This attribute is named `assocEntityType` in VSD API.
                
        """
        self._assoc_entity_type = value

    
    @property
    def associated_ip_filter_profile_id(self):
        """ Get associated_ip_filter_profile_id value.

            Notes:
                UUID of the associated IP Filter Profile entity.

                
                This attribute is named `associatedIPFilterProfileID` in VSD API.
                
        """
        return self._associated_ip_filter_profile_id

    @associated_ip_filter_profile_id.setter
    def associated_ip_filter_profile_id(self, value):
        """ Set associated_ip_filter_profile_id value.

            Notes:
                UUID of the associated IP Filter Profile entity.

                
                This attribute is named `associatedIPFilterProfileID` in VSD API.
                
        """
        self._associated_ip_filter_profile_id = value

    
    @property
    def associated_ip_filter_profile_name(self):
        """ Get associated_ip_filter_profile_name value.

            Notes:
                Name of the associated IP Filter Profile entity.

                
                This attribute is named `associatedIPFilterProfileName` in VSD API.
                
        """
        return self._associated_ip_filter_profile_name

    @associated_ip_filter_profile_name.setter
    def associated_ip_filter_profile_name(self, value):
        """ Set associated_ip_filter_profile_name value.

            Notes:
                Name of the associated IP Filter Profile entity.

                
                This attribute is named `associatedIPFilterProfileName` in VSD API.
                
        """
        self._associated_ip_filter_profile_name = value

    
    @property
    def associated_ipv6_filter_profile_id(self):
        """ Get associated_ipv6_filter_profile_id value.

            Notes:
                UUID of the associated IPv6 Filter Profile entity.

                
                This attribute is named `associatedIPv6FilterProfileID` in VSD API.
                
        """
        return self._associated_ipv6_filter_profile_id

    @associated_ipv6_filter_profile_id.setter
    def associated_ipv6_filter_profile_id(self, value):
        """ Set associated_ipv6_filter_profile_id value.

            Notes:
                UUID of the associated IPv6 Filter Profile entity.

                
                This attribute is named `associatedIPv6FilterProfileID` in VSD API.
                
        """
        self._associated_ipv6_filter_profile_id = value

    
    @property
    def associated_ipv6_filter_profile_name(self):
        """ Get associated_ipv6_filter_profile_name value.

            Notes:
                Name of the associated IPv6 Filter Profile entity.

                
                This attribute is named `associatedIPv6FilterProfileName` in VSD API.
                
        """
        return self._associated_ipv6_filter_profile_name

    @associated_ipv6_filter_profile_name.setter
    def associated_ipv6_filter_profile_name(self, value):
        """ Set associated_ipv6_filter_profile_name value.

            Notes:
                Name of the associated IPv6 Filter Profile entity.

                
                This attribute is named `associatedIPv6FilterProfileName` in VSD API.
                
        """
        self._associated_ipv6_filter_profile_name = value

    
    @property
    def associated_mac_filter_profile_id(self):
        """ Get associated_mac_filter_profile_id value.

            Notes:
                UUID of the associated MAC Filter Profile entity.

                
                This attribute is named `associatedMACFilterProfileID` in VSD API.
                
        """
        return self._associated_mac_filter_profile_id

    @associated_mac_filter_profile_id.setter
    def associated_mac_filter_profile_id(self, value):
        """ Set associated_mac_filter_profile_id value.

            Notes:
                UUID of the associated MAC Filter Profile entity.

                
                This attribute is named `associatedMACFilterProfileID` in VSD API.
                
        """
        self._associated_mac_filter_profile_id = value

    
    @property
    def associated_mac_filter_profile_name(self):
        """ Get associated_mac_filter_profile_name value.

            Notes:
                Name of the associated MAC Filter Profile entity.

                
                This attribute is named `associatedMACFilterProfileName` in VSD API.
                
        """
        return self._associated_mac_filter_profile_name

    @associated_mac_filter_profile_name.setter
    def associated_mac_filter_profile_name(self, value):
        """ Set associated_mac_filter_profile_name value.

            Notes:
                Name of the associated MAC Filter Profile entity.

                
                This attribute is named `associatedMACFilterProfileName` in VSD API.
                
        """
        self._associated_mac_filter_profile_name = value

    
    @property
    def associated_sap_egress_qo_s_profile_id(self):
        """ Get associated_sap_egress_qo_s_profile_id value.

            Notes:
                UUID of the associated SAP Egress QoS Profile entity.

                
                This attribute is named `associatedSAPEgressQoSProfileID` in VSD API.
                
        """
        return self._associated_sap_egress_qo_s_profile_id

    @associated_sap_egress_qo_s_profile_id.setter
    def associated_sap_egress_qo_s_profile_id(self, value):
        """ Set associated_sap_egress_qo_s_profile_id value.

            Notes:
                UUID of the associated SAP Egress QoS Profile entity.

                
                This attribute is named `associatedSAPEgressQoSProfileID` in VSD API.
                
        """
        self._associated_sap_egress_qo_s_profile_id = value

    
    @property
    def associated_sap_egress_qo_s_profile_name(self):
        """ Get associated_sap_egress_qo_s_profile_name value.

            Notes:
                Name of the associated SAP Egress QoS Profile entity.

                
                This attribute is named `associatedSAPEgressQoSProfileName` in VSD API.
                
        """
        return self._associated_sap_egress_qo_s_profile_name

    @associated_sap_egress_qo_s_profile_name.setter
    def associated_sap_egress_qo_s_profile_name(self, value):
        """ Set associated_sap_egress_qo_s_profile_name value.

            Notes:
                Name of the associated SAP Egress QoS Profile entity.

                
                This attribute is named `associatedSAPEgressQoSProfileName` in VSD API.
                
        """
        self._associated_sap_egress_qo_s_profile_name = value

    
    @property
    def external_id(self):
        """ Get external_id value.

            Notes:
                External object ID. Used for integration with third party systems

                
                This attribute is named `externalID` in VSD API.
                
        """
        return self._external_id

    @external_id.setter
    def external_id(self, value):
        """ Set external_id value.

            Notes:
                External object ID. Used for integration with third party systems

                
                This attribute is named `externalID` in VSD API.
                
        """
        self._external_id = value

    

    


# -*- coding: utf-8 -*-
#
# Copyright (c) 2015, Alcatel-Lucent Inc, 2017 Nokia
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the copyright holder nor the names of its contributors
#       may be used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.




from .fetchers import NUTCAsFetcher


from .fetchers import NURedirectionTargetsFetcher


from .fetchers import NUDeploymentFailuresFetcher


from .fetchers import NUMetadatasFetcher


from .fetchers import NUAggregateMetadatasFetcher


from .fetchers import NUBGPNeighborsFetcher


from .fetchers import NUEgressACLEntryTemplatesFetcher


from .fetchers import NUDHCPOptionsFetcher


from .fetchers import NUVirtualIPsFetcher


from .fetchers import NUAlarmsFetcher


from .fetchers import NUGlobalMetadatasFetcher


from .fetchers import NUVMsFetcher


from .fetchers import NUVMInterfacesFetcher


from .fetchers import NUVNFInterfacesFetcher


from .fetchers import NUIngressACLEntryTemplatesFetcher


from .fetchers import NUIngressAdvFwdEntryTemplatesFetcher


from .fetchers import NUJobsFetcher


from .fetchers import NUPolicyGroupsFetcher


from .fetchers import NUContainersFetcher


from .fetchers import NUContainerInterfacesFetcher


from .fetchers import NUPortMappingsFetcher


from .fetchers import NUQOSsFetcher


from .fetchers import NUHostInterfacesFetcher


from .fetchers import NUVPortMirrorsFetcher


from .fetchers import NUBridgeInterfacesFetcher


from .fetchers import NUVRSsFetcher


from .fetchers import NUTrunksFetcher


from .fetchers import NUStatisticsFetcher


from .fetchers import NUStatisticsPoliciesFetcher


from .fetchers import NUEventLogsFetcher

from bambou import NURESTObject


class NUVPort(NURESTObject):
    """ Represents a VPort in the VSD

        Notes:
            VPorts are a new level in the domain hierarchy, intended to provide more granular configuration than at subnet, and also support a split workflow, where the vPort is configured and associated with a VM port (or gateway port) before the port exists.
    """

    __rest_name__ = "vport"
    __resource_name__ = "vports"

    
    ## Constants
    
    CONST_SYSTEM_TYPE_NUAGE_2 = "NUAGE_2"
    
    CONST_SYSTEM_TYPE_NUAGE_1 = "NUAGE_1"
    
    CONST_SYSTEM_TYPE_HARDWARE_VTEP = "HARDWARE_VTEP"
    
    CONST_ASSOCIATED_GATEWAY_PERSONALITY_VDFG = "VDFG"
    
    CONST_ASSOCIATED_GATEWAY_PERSONALITY_EVDFB = "EVDFB"
    
    CONST_DPI_ENABLED = "ENABLED"
    
    CONST_ENTITY_SCOPE_ENTERPRISE = "ENTERPRISE"
    
    CONST_TYPE_CONTAINER = "CONTAINER"
    
    CONST_DPI_INHERITED = "INHERITED"
    
    CONST_ASSOCIATED_GATEWAY_PERSONALITY_DC7X50 = "DC7X50"
    
    CONST_ADDRESS_SPOOFING_DISABLED = "DISABLED"
    
    CONST_ADDRESS_SPOOFING_INHERITED = "INHERITED"
    
    CONST_TYPE_BRIDGE = "BRIDGE"
    
    CONST_ADDRESS_SPOOFING_ENABLED = "ENABLED"
    
    CONST_ASSOCIATED_GATEWAY_PERSONALITY_NUAGE_210_WBX_48_S = "NUAGE_210_WBX_48_S"
    
    CONST_PEER_OPERATIONAL_STATE_INIT = "INIT"
    
    CONST_MULTICAST_DISABLED = "DISABLED"
    
    CONST_ASSOCIATED_GATEWAY_PERSONALITY_VSG = "VSG"
    
    CONST_ASSOCIATED_GATEWAY_PERSONALITY_NUAGE_210_WBX_32_Q = "NUAGE_210_WBX_32_Q"
    
    CONST_ASSOCIATED_GATEWAY_PERSONALITY_VSA = "VSA"
    
    CONST_ASSOCIATED_GATEWAY_PERSONALITY_NSGDUC = "NSGDUC"
    
    CONST_ASSOCIATED_GATEWAY_PERSONALITY_NSG = "NSG"
    
    CONST_TYPE_HOST = "HOST"
    
    CONST_GATEWAY_MAC_MOVE_ROLE_TERTIARY = "TERTIARY"
    
    CONST_GATEWAY_MAC_MOVE_ROLE_SECONDARY = "SECONDARY"
    
    CONST_SUB_TYPE_VNF = "VNF"
    
    CONST_OPERATIONAL_STATE_DOWN = "DOWN"
    
    CONST_ASSOCIATED_GATEWAY_PERSONALITY_OTHER = "OTHER"
    
    CONST_PEER_OPERATIONAL_STATE_DOWN = "DOWN"
    
    CONST_TRUNK_ROLE_SUB_PORT = "SUB_PORT"
    
    CONST_SEGMENTATION_TYPE_VLAN = "VLAN"
    
    CONST_OPERATIONAL_STATE_UP = "UP"
    
    CONST_ASSOCIATED_GATEWAY_PERSONALITY_VDF = "VDF"
    
    CONST_PEER_OPERATIONAL_STATE_UP = "UP"
    
    CONST_MULTICAST_ENABLED = "ENABLED"
    
    CONST_MULTICAST_INHERITED = "INHERITED"
    
    CONST_FIP_IGNORE_DEFAULT_ROUTE_DISABLED = "DISABLED"
    
    CONST_FIP_IGNORE_DEFAULT_ROUTE_INHERITED = "INHERITED"
    
    CONST_SYSTEM_TYPE_HARDWARE = "HARDWARE"
    
    CONST_ASSOCIATED_GATEWAY_PERSONALITY_HARDWARE_VTEP = "HARDWARE_VTEP"
    
    CONST_ASSOCIATED_GATEWAY_PERSONALITY_VRSB = "VRSB"
    
    CONST_ASSOCIATED_GATEWAY_PERSONALITY_VRSG = "VRSG"
    
    CONST_SUB_TYPE_NONE = "NONE"
    
    CONST_SYSTEM_TYPE_SOFTWARE = "SOFTWARE"
    
    CONST_ASSOCIATED_GATEWAY_PERSONALITY_EVDF = "EVDF"
    
    CONST_ENTITY_SCOPE_GLOBAL = "GLOBAL"
    
    CONST_ASSOCIATED_GATEWAY_PERSONALITY_NETCONF_THIRDPARTY_HW_VTEP = "NETCONF_THIRDPARTY_HW_VTEP"
    
    CONST_TRUNK_ROLE_PARENT_PORT = "PARENT_PORT"
    
    CONST_ASSOCIATED_GATEWAY_PERSONALITY_NETCONF_7X50 = "NETCONF_7X50"
    
    CONST_DPI_DISABLED = "DISABLED"
    
    CONST_ASSOCIATED_GATEWAY_PERSONALITY_NSGBR = "NSGBR"
    
    CONST_SYSTEM_TYPE_NUAGE_VRSG = "NUAGE_VRSG"
    
    CONST_OPERATIONAL_STATE_INIT = "INIT"
    
    CONST_TYPE_VM = "VM"
    
    CONST_FIP_IGNORE_DEFAULT_ROUTE_ENABLED = "ENABLED"
    
    

    def __init__(self, **kwargs):
        """ Initializes a VPort instance

            Notes:
                You can specify all parameters while calling this methods.
                A special argument named `data` will enable you to load the
                object from a Python dictionary

            Examples:
                >>> vport = NUVPort(id=u'xxxx-xxx-xxx-xxx', name=u'VPort')
                >>> vport = NUVPort(data=my_dict)
        """

        super(NUVPort, self).__init__()

        # Read/Write Attributes
        
        self._fip_ignore_default_route = None
        self._vlan = None
        self._vlanid = None
        self._dpi = None
        self._backhaul_subnet_vnid = None
        self._name = None
        self._has_attached_interfaces = None
        self._last_updated_by = None
        self._gateway_mac_move_role = None
        self._gateway_port_name = None
        self._access_restriction_enabled = None
        self._active = None
        self._address_spoofing = None
        self._peer_operational_state = None
        self._segmentation_id = None
        self._segmentation_type = None
        self._service_id = None
        self._description = None
        self._entity_scope = None
        self._domain_id = None
        self._domain_name = None
        self._domain_service_label = None
        self._domain_vlanid = None
        self._zone_id = None
        self._operational_state = None
        self._trunk_role = None
        self._assoc_entity_id = None
        self._associated_egress_profile_id = None
        self._associated_floating_ip_id = None
        self._associated_gateway_id = None
        self._associated_gateway_personality = None
        self._associated_gateway_type = None
        self._associated_ingress_profile_id = None
        self._associated_multicast_channel_map_id = None
        self._associated_ssid = None
        self._associated_send_multicast_channel_map_id = None
        self._associated_trunk_id = None
        self._sub_type = None
        self._subnet_vnid = None
        self._multi_nic_vport_id = None
        self._multicast = None
        self._gw_eligible = None
        self._external_id = None
        self._type = None
        self._system_type = None
        
        self.expose_attribute(local_name="fip_ignore_default_route", remote_name="FIPIgnoreDefaultRoute", attribute_type=str, is_required=False, is_unique=False, choices=[u'DISABLED', u'ENABLED', u'INHERITED'])
        self.expose_attribute(local_name="vlan", remote_name="VLAN", attribute_type=int, is_required=False, is_unique=False)
        self.expose_attribute(local_name="vlanid", remote_name="VLANID", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="dpi", remote_name="DPI", attribute_type=str, is_required=False, is_unique=False, choices=[u'DISABLED', u'ENABLED', u'INHERITED'])
        self.expose_attribute(local_name="backhaul_subnet_vnid", remote_name="backhaulSubnetVNID", attribute_type=int, is_required=False, is_unique=False)
        self.expose_attribute(local_name="name", remote_name="name", attribute_type=str, is_required=True, is_unique=False)
        self.expose_attribute(local_name="has_attached_interfaces", remote_name="hasAttachedInterfaces", attribute_type=bool, is_required=False, is_unique=False)
        self.expose_attribute(local_name="last_updated_by", remote_name="lastUpdatedBy", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="gateway_mac_move_role", remote_name="gatewayMACMoveRole", attribute_type=str, is_required=False, is_unique=False, choices=[u'SECONDARY', u'TERTIARY'])
        self.expose_attribute(local_name="gateway_port_name", remote_name="gatewayPortName", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="access_restriction_enabled", remote_name="accessRestrictionEnabled", attribute_type=bool, is_required=False, is_unique=False)
        self.expose_attribute(local_name="active", remote_name="active", attribute_type=bool, is_required=False, is_unique=False)
        self.expose_attribute(local_name="address_spoofing", remote_name="addressSpoofing", attribute_type=str, is_required=True, is_unique=False, choices=[u'DISABLED', u'ENABLED', u'INHERITED'])
        self.expose_attribute(local_name="peer_operational_state", remote_name="peerOperationalState", attribute_type=str, is_required=False, is_unique=False, choices=[u'DOWN', u'INIT', u'UP'])
        self.expose_attribute(local_name="segmentation_id", remote_name="segmentationID", attribute_type=int, is_required=False, is_unique=False)
        self.expose_attribute(local_name="segmentation_type", remote_name="segmentationType", attribute_type=str, is_required=False, is_unique=False, choices=[u'VLAN'])
        self.expose_attribute(local_name="service_id", remote_name="serviceID", attribute_type=int, is_required=False, is_unique=False)
        self.expose_attribute(local_name="description", remote_name="description", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="entity_scope", remote_name="entityScope", attribute_type=str, is_required=False, is_unique=False, choices=[u'ENTERPRISE', u'GLOBAL'])
        self.expose_attribute(local_name="domain_id", remote_name="domainID", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="domain_name", remote_name="domainName", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="domain_service_label", remote_name="domainServiceLabel", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="domain_vlanid", remote_name="domainVLANID", attribute_type=int, is_required=False, is_unique=False)
        self.expose_attribute(local_name="zone_id", remote_name="zoneID", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="operational_state", remote_name="operationalState", attribute_type=str, is_required=False, is_unique=False, choices=[u'DOWN', u'INIT', u'UP'])
        self.expose_attribute(local_name="trunk_role", remote_name="trunkRole", attribute_type=str, is_required=False, is_unique=False, choices=[u'PARENT_PORT', u'SUB_PORT'])
        self.expose_attribute(local_name="assoc_entity_id", remote_name="assocEntityID", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="associated_egress_profile_id", remote_name="associatedEgressProfileID", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="associated_floating_ip_id", remote_name="associatedFloatingIPID", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="associated_gateway_id", remote_name="associatedGatewayID", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="associated_gateway_personality", remote_name="associatedGatewayPersonality", attribute_type=str, is_required=False, is_unique=False, choices=[u'DC7X50', u'EVDF', u'EVDFB', u'HARDWARE_VTEP', u'NETCONF_7X50', u'NETCONF_THIRDPARTY_HW_VTEP', u'NSG', u'NSGBR', u'NSGDUC', u'NUAGE_210_WBX_32_Q', u'NUAGE_210_WBX_48_S', u'OTHER', u'VDF', u'VDFG', u'VRSB', u'VRSG', u'VSA', u'VSG'])
        self.expose_attribute(local_name="associated_gateway_type", remote_name="associatedGatewayType", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="associated_ingress_profile_id", remote_name="associatedIngressProfileID", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="associated_multicast_channel_map_id", remote_name="associatedMulticastChannelMapID", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="associated_ssid", remote_name="associatedSSID", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="associated_send_multicast_channel_map_id", remote_name="associatedSendMulticastChannelMapID", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="associated_trunk_id", remote_name="associatedTrunkID", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="sub_type", remote_name="subType", attribute_type=str, is_required=False, is_unique=False, choices=[u'NONE', u'VNF'])
        self.expose_attribute(local_name="subnet_vnid", remote_name="subnetVNID", attribute_type=int, is_required=False, is_unique=False)
        self.expose_attribute(local_name="multi_nic_vport_id", remote_name="multiNICVPortID", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="multicast", remote_name="multicast", attribute_type=str, is_required=False, is_unique=False, choices=[u'DISABLED', u'ENABLED', u'INHERITED'])
        self.expose_attribute(local_name="gw_eligible", remote_name="gwEligible", attribute_type=bool, is_required=False, is_unique=False)
        self.expose_attribute(local_name="external_id", remote_name="externalID", attribute_type=str, is_required=False, is_unique=True)
        self.expose_attribute(local_name="type", remote_name="type", attribute_type=str, is_required=True, is_unique=False, choices=[u'BRIDGE', u'CONTAINER', u'HOST', u'VM'])
        self.expose_attribute(local_name="system_type", remote_name="systemType", attribute_type=str, is_required=False, is_unique=False, choices=[u'HARDWARE', u'HARDWARE_VTEP', u'NUAGE_1', u'NUAGE_2', u'NUAGE_VRSG', u'SOFTWARE'])
        

        # Fetchers
        
        
        self.tcas = NUTCAsFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.redirection_targets = NURedirectionTargetsFetcher.fetcher_with_object(parent_object=self, relationship="member")
        
        
        self.deployment_failures = NUDeploymentFailuresFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.metadatas = NUMetadatasFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.aggregate_metadatas = NUAggregateMetadatasFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.bgp_neighbors = NUBGPNeighborsFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.egress_acl_entry_templates = NUEgressACLEntryTemplatesFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.dhcp_options = NUDHCPOptionsFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.virtual_ips = NUVirtualIPsFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.alarms = NUAlarmsFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.global_metadatas = NUGlobalMetadatasFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.vms = NUVMsFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.vm_interfaces = NUVMInterfacesFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.vnf_interfaces = NUVNFInterfacesFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.ingress_acl_entry_templates = NUIngressACLEntryTemplatesFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.ingress_adv_fwd_entry_templates = NUIngressAdvFwdEntryTemplatesFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.jobs = NUJobsFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.policy_groups = NUPolicyGroupsFetcher.fetcher_with_object(parent_object=self, relationship="member")
        
        
        self.containers = NUContainersFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.container_interfaces = NUContainerInterfacesFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.port_mappings = NUPortMappingsFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.qoss = NUQOSsFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.host_interfaces = NUHostInterfacesFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.vport_mirrors = NUVPortMirrorsFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.bridge_interfaces = NUBridgeInterfacesFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.vrss = NUVRSsFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.trunks = NUTrunksFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.statistics = NUStatisticsFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.statistics_policies = NUStatisticsPoliciesFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.event_logs = NUEventLogsFetcher.fetcher_with_object(parent_object=self, relationship="child")
        

        self._compute_args(**kwargs)

    # Properties
    
    @property
    def fip_ignore_default_route(self):
        """ Get fip_ignore_default_route value.

            Notes:
                Determines whether the default Overlay route will be ignored or not when a VM has FIP so that it takes Underlay route.

                
                This attribute is named `FIPIgnoreDefaultRoute` in VSD API.
                
        """
        return self._fip_ignore_default_route

    @fip_ignore_default_route.setter
    def fip_ignore_default_route(self, value):
        """ Set fip_ignore_default_route value.

            Notes:
                Determines whether the default Overlay route will be ignored or not when a VM has FIP so that it takes Underlay route.

                
                This attribute is named `FIPIgnoreDefaultRoute` in VSD API.
                
        """
        self._fip_ignore_default_route = value

    
    @property
    def vlan(self):
        """ Get vlan value.

            Notes:
                VLAN number of the associated VLAN of this vport - applicable for type host or bridge

                
                This attribute is named `VLAN` in VSD API.
                
        """
        return self._vlan

    @vlan.setter
    def vlan(self, value):
        """ Set vlan value.

            Notes:
                VLAN number of the associated VLAN of this vport - applicable for type host or bridge

                
                This attribute is named `VLAN` in VSD API.
                
        """
        self._vlan = value

    
    @property
    def vlanid(self):
        """ Get vlanid value.

            Notes:
                UUID of the associated VLAN of this vport - applicable for type host or bridge

                
                This attribute is named `VLANID` in VSD API.
                
        """
        return self._vlanid

    @vlanid.setter
    def vlanid(self, value):
        """ Set vlanid value.

            Notes:
                UUID of the associated VLAN of this vport - applicable for type host or bridge

                
                This attribute is named `VLANID` in VSD API.
                
        """
        self._vlanid = value

    
    @property
    def dpi(self):
        """ Get dpi value.

            Notes:
                determines whether or not deep packet inspection is enabled

                
                This attribute is named `DPI` in VSD API.
                
        """
        return self._dpi

    @dpi.setter
    def dpi(self, value):
        """ Set dpi value.

            Notes:
                determines whether or not deep packet inspection is enabled

                
                This attribute is named `DPI` in VSD API.
                
        """
        self._dpi = value

    
    @property
    def backhaul_subnet_vnid(self):
        """ Get backhaul_subnet_vnid value.

            Notes:
                Backhaul subnet VNID of the L3Domain associated with the VPort. This is exposed for Netconf manager

                
                This attribute is named `backhaulSubnetVNID` in VSD API.
                
        """
        return self._backhaul_subnet_vnid

    @backhaul_subnet_vnid.setter
    def backhaul_subnet_vnid(self, value):
        """ Set backhaul_subnet_vnid value.

            Notes:
                Backhaul subnet VNID of the L3Domain associated with the VPort. This is exposed for Netconf manager

                
                This attribute is named `backhaulSubnetVNID` in VSD API.
                
        """
        self._backhaul_subnet_vnid = value

    
    @property
    def name(self):
        """ Get name value.

            Notes:
                Name of the vport. Valid characters are alphabets, numbers, space and hyphen( - ).

                
        """
        return self._name

    @name.setter
    def name(self, value):
        """ Set name value.

            Notes:
                Name of the vport. Valid characters are alphabets, numbers, space and hyphen( - ).

                
        """
        self._name = value

    
    @property
    def has_attached_interfaces(self):
        """ Get has_attached_interfaces value.

            Notes:
                Indicates that this vport has attached interfaces

                
                This attribute is named `hasAttachedInterfaces` in VSD API.
                
        """
        return self._has_attached_interfaces

    @has_attached_interfaces.setter
    def has_attached_interfaces(self, value):
        """ Set has_attached_interfaces value.

            Notes:
                Indicates that this vport has attached interfaces

                
                This attribute is named `hasAttachedInterfaces` in VSD API.
                
        """
        self._has_attached_interfaces = value

    
    @property
    def last_updated_by(self):
        """ Get last_updated_by value.

            Notes:
                ID of the user who last updated the object.

                
                This attribute is named `lastUpdatedBy` in VSD API.
                
        """
        return self._last_updated_by

    @last_updated_by.setter
    def last_updated_by(self, value):
        """ Set last_updated_by value.

            Notes:
                ID of the user who last updated the object.

                
                This attribute is named `lastUpdatedBy` in VSD API.
                
        """
        self._last_updated_by = value

    
    @property
    def gateway_mac_move_role(self):
        """ Get gateway_mac_move_role value.

            Notes:
                Role of the gateway vport when handling MAC move errors

                
                This attribute is named `gatewayMACMoveRole` in VSD API.
                
        """
        return self._gateway_mac_move_role

    @gateway_mac_move_role.setter
    def gateway_mac_move_role(self, value):
        """ Set gateway_mac_move_role value.

            Notes:
                Role of the gateway vport when handling MAC move errors

                
                This attribute is named `gatewayMACMoveRole` in VSD API.
                
        """
        self._gateway_mac_move_role = value

    
    @property
    def gateway_port_name(self):
        """ Get gateway_port_name value.

            Notes:
                Gateway portname eg: eth1 - applicable for type host/bridge

                
                This attribute is named `gatewayPortName` in VSD API.
                
        """
        return self._gateway_port_name

    @gateway_port_name.setter
    def gateway_port_name(self, value):
        """ Set gateway_port_name value.

            Notes:
                Gateway portname eg: eth1 - applicable for type host/bridge

                
                This attribute is named `gatewayPortName` in VSD API.
                
        """
        self._gateway_port_name = value

    
    @property
    def access_restriction_enabled(self):
        """ Get access_restriction_enabled value.

            Notes:
                Enable Access Restriction

                
                This attribute is named `accessRestrictionEnabled` in VSD API.
                
        """
        return self._access_restriction_enabled

    @access_restriction_enabled.setter
    def access_restriction_enabled(self, value):
        """ Set access_restriction_enabled value.

            Notes:
                Enable Access Restriction

                
                This attribute is named `accessRestrictionEnabled` in VSD API.
                
        """
        self._access_restriction_enabled = value

    
    @property
    def active(self):
        """ Get active value.

            Notes:
                Indicates if this vport is up or down

                
        """
        return self._active

    @active.setter
    def active(self, value):
        """ Set active value.

            Notes:
                Indicates if this vport is up or down

                
        """
        self._active = value

    
    @property
    def address_spoofing(self):
        """ Get address_spoofing value.

            Notes:
                Indicates if address spoofing is ENABLED/DISABLED/INHERITED for this vport.

                
                This attribute is named `addressSpoofing` in VSD API.
                
        """
        return self._address_spoofing

    @address_spoofing.setter
    def address_spoofing(self, value):
        """ Set address_spoofing value.

            Notes:
                Indicates if address spoofing is ENABLED/DISABLED/INHERITED for this vport.

                
                This attribute is named `addressSpoofing` in VSD API.
                
        """
        self._address_spoofing = value

    
    @property
    def peer_operational_state(self):
        """ Get peer_operational_state value.

            Notes:
                Operational state of the peer vport in multichassis lag scenario

                
                This attribute is named `peerOperationalState` in VSD API.
                
        """
        return self._peer_operational_state

    @peer_operational_state.setter
    def peer_operational_state(self, value):
        """ Set peer_operational_state value.

            Notes:
                Operational state of the peer vport in multichassis lag scenario

                
                This attribute is named `peerOperationalState` in VSD API.
                
        """
        self._peer_operational_state = value

    
    @property
    def segmentation_id(self):
        """ Get segmentation_id value.

            Notes:
                The VLAN Number (1-4095), valid only if the trunkRole is SUB_PORT

                
                This attribute is named `segmentationID` in VSD API.
                
        """
        return self._segmentation_id

    @segmentation_id.setter
    def segmentation_id(self, value):
        """ Set segmentation_id value.

            Notes:
                The VLAN Number (1-4095), valid only if the trunkRole is SUB_PORT

                
                This attribute is named `segmentationID` in VSD API.
                
        """
        self._segmentation_id = value

    
    @property
    def segmentation_type(self):
        """ Get segmentation_type value.

            Notes:
                The type of segmentation that is used. This must be VLAN for vports with trunkRole set to SUB_PORT. This can not be specified for a parent vport (trunkRole = PARENT_PORT)

                
                This attribute is named `segmentationType` in VSD API.
                
        """
        return self._segmentation_type

    @segmentation_type.setter
    def segmentation_type(self, value):
        """ Set segmentation_type value.

            Notes:
                The type of segmentation that is used. This must be VLAN for vports with trunkRole set to SUB_PORT. This can not be specified for a parent vport (trunkRole = PARENT_PORT)

                
                This attribute is named `segmentationType` in VSD API.
                
        """
        self._segmentation_type = value

    
    @property
    def service_id(self):
        """ Get service_id value.

            Notes:
                The service ID used by the VSCs to identify the subnet associated with this vport

                
                This attribute is named `serviceID` in VSD API.
                
        """
        return self._service_id

    @service_id.setter
    def service_id(self, value):
        """ Set service_id value.

            Notes:
                The service ID used by the VSCs to identify the subnet associated with this vport

                
                This attribute is named `serviceID` in VSD API.
                
        """
        self._service_id = value

    
    @property
    def description(self):
        """ Get description value.

            Notes:
                Description for this vport

                
        """
        return self._description

    @description.setter
    def description(self, value):
        """ Set description value.

            Notes:
                Description for this vport

                
        """
        self._description = value

    
    @property
    def entity_scope(self):
        """ Get entity_scope value.

            Notes:
                Specify if scope of entity is Data center or Enterprise level

                
                This attribute is named `entityScope` in VSD API.
                
        """
        return self._entity_scope

    @entity_scope.setter
    def entity_scope(self, value):
        """ Set entity_scope value.

            Notes:
                Specify if scope of entity is Data center or Enterprise level

                
                This attribute is named `entityScope` in VSD API.
                
        """
        self._entity_scope = value

    
    @property
    def domain_id(self):
        """ Get domain_id value.

            Notes:
                ID the Domain associated with the VPort

                
                This attribute is named `domainID` in VSD API.
                
        """
        return self._domain_id

    @domain_id.setter
    def domain_id(self, value):
        """ Set domain_id value.

            Notes:
                ID the Domain associated with the VPort

                
                This attribute is named `domainID` in VSD API.
                
        """
        self._domain_id = value

    
    @property
    def domain_name(self):
        """ Get domain_name value.

            Notes:
                Name of the Domain associated with the VPort. This is exposed for Netconf manager 

                
                This attribute is named `domainName` in VSD API.
                
        """
        return self._domain_name

    @domain_name.setter
    def domain_name(self, value):
        """ Set domain_name value.

            Notes:
                Name of the Domain associated with the VPort. This is exposed for Netconf manager 

                
                This attribute is named `domainName` in VSD API.
                
        """
        self._domain_name = value

    
    @property
    def domain_service_label(self):
        """ Get domain_service_label value.

            Notes:
                Service ID of Domain.

                
                This attribute is named `domainServiceLabel` in VSD API.
                
        """
        return self._domain_service_label

    @domain_service_label.setter
    def domain_service_label(self, value):
        """ Set domain_service_label value.

            Notes:
                Service ID of Domain.

                
                This attribute is named `domainServiceLabel` in VSD API.
                
        """
        self._domain_service_label = value

    
    @property
    def domain_vlanid(self):
        """ Get domain_vlanid value.

            Notes:
                Backhaul vlan id the L3Domain associated with the VPort. This is exposed for Netconf manager

                
                This attribute is named `domainVLANID` in VSD API.
                
        """
        return self._domain_vlanid

    @domain_vlanid.setter
    def domain_vlanid(self, value):
        """ Set domain_vlanid value.

            Notes:
                Backhaul vlan id the L3Domain associated with the VPort. This is exposed for Netconf manager

                
                This attribute is named `domainVLANID` in VSD API.
                
        """
        self._domain_vlanid = value

    
    @property
    def zone_id(self):
        """ Get zone_id value.

            Notes:
                ID the Zone associated with the VPort

                
                This attribute is named `zoneID` in VSD API.
                
        """
        return self._zone_id

    @zone_id.setter
    def zone_id(self, value):
        """ Set zone_id value.

            Notes:
                ID the Zone associated with the VPort

                
                This attribute is named `zoneID` in VSD API.
                
        """
        self._zone_id = value

    
    @property
    def operational_state(self):
        """ Get operational_state value.

            Notes:
                Operational State of the VPort. Possible values are INIT, UP, DOWN.

                
                This attribute is named `operationalState` in VSD API.
                
        """
        return self._operational_state

    @operational_state.setter
    def operational_state(self, value):
        """ Set operational_state value.

            Notes:
                Operational State of the VPort. Possible values are INIT, UP, DOWN.

                
                This attribute is named `operationalState` in VSD API.
                
        """
        self._operational_state = value

    
    @property
    def trunk_role(self):
        """ Get trunk_role value.

            Notes:
                Indicates the role of the vport in trunking operations

                
                This attribute is named `trunkRole` in VSD API.
                
        """
        return self._trunk_role

    @trunk_role.setter
    def trunk_role(self, value):
        """ Set trunk_role value.

            Notes:
                Indicates the role of the vport in trunking operations

                
                This attribute is named `trunkRole` in VSD API.
                
        """
        self._trunk_role = value

    
    @property
    def assoc_entity_id(self):
        """ Get assoc_entity_id value.

            Notes:
                UUID of the entity to which the vport is associated to. This could be UUID of a SUBNET or a L2DOMAIN

                
                This attribute is named `assocEntityID` in VSD API.
                
        """
        return self._assoc_entity_id

    @assoc_entity_id.setter
    def assoc_entity_id(self, value):
        """ Set assoc_entity_id value.

            Notes:
                UUID of the entity to which the vport is associated to. This could be UUID of a SUBNET or a L2DOMAIN

                
                This attribute is named `assocEntityID` in VSD API.
                
        """
        self._assoc_entity_id = value

    
    @property
    def associated_egress_profile_id(self):
        """ Get associated_egress_profile_id value.

            Notes:
                UUID of the Egress Profile associated with this Vport entity.

                
                This attribute is named `associatedEgressProfileID` in VSD API.
                
        """
        return self._associated_egress_profile_id

    @associated_egress_profile_id.setter
    def associated_egress_profile_id(self, value):
        """ Set associated_egress_profile_id value.

            Notes:
                UUID of the Egress Profile associated with this Vport entity.

                
                This attribute is named `associatedEgressProfileID` in VSD API.
                
        """
        self._associated_egress_profile_id = value

    
    @property
    def associated_floating_ip_id(self):
        """ Get associated_floating_ip_id value.

            Notes:
                Id of Floating IP address associated to this vport

                
                This attribute is named `associatedFloatingIPID` in VSD API.
                
        """
        return self._associated_floating_ip_id

    @associated_floating_ip_id.setter
    def associated_floating_ip_id(self, value):
        """ Set associated_floating_ip_id value.

            Notes:
                Id of Floating IP address associated to this vport

                
                This attribute is named `associatedFloatingIPID` in VSD API.
                
        """
        self._associated_floating_ip_id = value

    
    @property
    def associated_gateway_id(self):
        """ Get associated_gateway_id value.

            Notes:
                Associated gateway ID of VPort

                
                This attribute is named `associatedGatewayID` in VSD API.
                
        """
        return self._associated_gateway_id

    @associated_gateway_id.setter
    def associated_gateway_id(self, value):
        """ Set associated_gateway_id value.

            Notes:
                Associated gateway ID of VPort

                
                This attribute is named `associatedGatewayID` in VSD API.
                
        """
        self._associated_gateway_id = value

    
    @property
    def associated_gateway_personality(self):
        """ Get associated_gateway_personality value.

            Notes:
                Personality of the associated Gateway

                
                This attribute is named `associatedGatewayPersonality` in VSD API.
                
        """
        return self._associated_gateway_personality

    @associated_gateway_personality.setter
    def associated_gateway_personality(self, value):
        """ Set associated_gateway_personality value.

            Notes:
                Personality of the associated Gateway

                
                This attribute is named `associatedGatewayPersonality` in VSD API.
                
        """
        self._associated_gateway_personality = value

    
    @property
    def associated_gateway_type(self):
        """ Get associated_gateway_type value.

            Notes:
                Associated gateway type of VPort.

                
                This attribute is named `associatedGatewayType` in VSD API.
                
        """
        return self._associated_gateway_type

    @associated_gateway_type.setter
    def associated_gateway_type(self, value):
        """ Set associated_gateway_type value.

            Notes:
                Associated gateway type of VPort.

                
                This attribute is named `associatedGatewayType` in VSD API.
                
        """
        self._associated_gateway_type = value

    
    @property
    def associated_ingress_profile_id(self):
        """ Get associated_ingress_profile_id value.

            Notes:
                UUID of the Ingress Profile associated with this Vport entity.

                
                This attribute is named `associatedIngressProfileID` in VSD API.
                
        """
        return self._associated_ingress_profile_id

    @associated_ingress_profile_id.setter
    def associated_ingress_profile_id(self, value):
        """ Set associated_ingress_profile_id value.

            Notes:
                UUID of the Ingress Profile associated with this Vport entity.

                
                This attribute is named `associatedIngressProfileID` in VSD API.
                
        """
        self._associated_ingress_profile_id = value

    
    @property
    def associated_multicast_channel_map_id(self):
        """ Get associated_multicast_channel_map_id value.

            Notes:
                The ID of the receive Multicast Channel Map this Vport is associated with. This has to be set when enableMultiCast is set to ENABLED

                
                This attribute is named `associatedMulticastChannelMapID` in VSD API.
                
        """
        return self._associated_multicast_channel_map_id

    @associated_multicast_channel_map_id.setter
    def associated_multicast_channel_map_id(self, value):
        """ Set associated_multicast_channel_map_id value.

            Notes:
                The ID of the receive Multicast Channel Map this Vport is associated with. This has to be set when enableMultiCast is set to ENABLED

                
                This attribute is named `associatedMulticastChannelMapID` in VSD API.
                
        """
        self._associated_multicast_channel_map_id = value

    
    @property
    def associated_ssid(self):
        """ Get associated_ssid value.

            Notes:
                The UUID of the SSID Connection tied to this instance of a vPort.

                
                This attribute is named `associatedSSID` in VSD API.
                
        """
        return self._associated_ssid

    @associated_ssid.setter
    def associated_ssid(self, value):
        """ Set associated_ssid value.

            Notes:
                The UUID of the SSID Connection tied to this instance of a vPort.

                
                This attribute is named `associatedSSID` in VSD API.
                
        """
        self._associated_ssid = value

    
    @property
    def associated_send_multicast_channel_map_id(self):
        """ Get associated_send_multicast_channel_map_id value.

            Notes:
                The ID of the send Multicast Channel Map this Vport is associated with. This has to be set when enableMultiCast is set to ENABLED

                
                This attribute is named `associatedSendMulticastChannelMapID` in VSD API.
                
        """
        return self._associated_send_multicast_channel_map_id

    @associated_send_multicast_channel_map_id.setter
    def associated_send_multicast_channel_map_id(self, value):
        """ Set associated_send_multicast_channel_map_id value.

            Notes:
                The ID of the send Multicast Channel Map this Vport is associated with. This has to be set when enableMultiCast is set to ENABLED

                
                This attribute is named `associatedSendMulticastChannelMapID` in VSD API.
                
        """
        self._associated_send_multicast_channel_map_id = value

    
    @property
    def associated_trunk_id(self):
        """ Get associated_trunk_id value.

            Notes:
                The trunk uuid associated with another vport of trunkRole PARENT_PORT. Can be specified only if trunkRole of this vport is SUB_PORT.

                
                This attribute is named `associatedTrunkID` in VSD API.
                
        """
        return self._associated_trunk_id

    @associated_trunk_id.setter
    def associated_trunk_id(self, value):
        """ Set associated_trunk_id value.

            Notes:
                The trunk uuid associated with another vport of trunkRole PARENT_PORT. Can be specified only if trunkRole of this vport is SUB_PORT.

                
                This attribute is named `associatedTrunkID` in VSD API.
                
        """
        self._associated_trunk_id = value

    
    @property
    def sub_type(self):
        """ Get sub_type value.

            Notes:
                Sub type of vport - possible values are NONE/VNF

                
                This attribute is named `subType` in VSD API.
                
        """
        return self._sub_type

    @sub_type.setter
    def sub_type(self, value):
        """ Set sub_type value.

            Notes:
                Sub type of vport - possible values are NONE/VNF

                
                This attribute is named `subType` in VSD API.
                
        """
        self._sub_type = value

    
    @property
    def subnet_vnid(self):
        """ Get subnet_vnid value.

            Notes:
                VNID of the associated subnet or L2domain with the VPort. This is exposed for Netconf manager

                
                This attribute is named `subnetVNID` in VSD API.
                
        """
        return self._subnet_vnid

    @subnet_vnid.setter
    def subnet_vnid(self, value):
        """ Set subnet_vnid value.

            Notes:
                VNID of the associated subnet or L2domain with the VPort. This is exposed for Netconf manager

                
                This attribute is named `subnetVNID` in VSD API.
                
        """
        self._subnet_vnid = value

    
    @property
    def multi_nic_vport_id(self):
        """ Get multi_nic_vport_id value.

            Notes:
                ID of the Multi NIC VPort associated with the VPort

                
                This attribute is named `multiNICVPortID` in VSD API.
                
        """
        return self._multi_nic_vport_id

    @multi_nic_vport_id.setter
    def multi_nic_vport_id(self, value):
        """ Set multi_nic_vport_id value.

            Notes:
                ID of the Multi NIC VPort associated with the VPort

                
                This attribute is named `multiNICVPortID` in VSD API.
                
        """
        self._multi_nic_vport_id = value

    
    @property
    def multicast(self):
        """ Get multicast value.

            Notes:
                Indicates multicast policy on Vport.

                
        """
        return self._multicast

    @multicast.setter
    def multicast(self, value):
        """ Set multicast value.

            Notes:
                Indicates multicast policy on Vport.

                
        """
        self._multicast = value

    
    @property
    def gw_eligible(self):
        """ Get gw_eligible value.

            Notes:
                Indicates that this vport is eligible to be given in gateway vport config request. It becomes eligible when it has properly attached host or bridge interfaces.

                
                This attribute is named `gwEligible` in VSD API.
                
        """
        return self._gw_eligible

    @gw_eligible.setter
    def gw_eligible(self, value):
        """ Set gw_eligible value.

            Notes:
                Indicates that this vport is eligible to be given in gateway vport config request. It becomes eligible when it has properly attached host or bridge interfaces.

                
                This attribute is named `gwEligible` in VSD API.
                
        """
        self._gw_eligible = value

    
    @property
    def external_id(self):
        """ Get external_id value.

            Notes:
                External object ID. Used for integration with third party systems

                
                This attribute is named `externalID` in VSD API.
                
        """
        return self._external_id

    @external_id.setter
    def external_id(self, value):
        """ Set external_id value.

            Notes:
                External object ID. Used for integration with third party systems

                
                This attribute is named `externalID` in VSD API.
                
        """
        self._external_id = value

    
    @property
    def type(self):
        """ Get type value.

            Notes:
                Type of vport. Possible values are VM, HOST, BRIDGE, CONTAINER.

                
        """
        return self._type

    @type.setter
    def type(self, value):
        """ Set type value.

            Notes:
                Type of vport. Possible values are VM, HOST, BRIDGE, CONTAINER.

                
        """
        self._type = value

    
    @property
    def system_type(self):
        """ Get system_type value.

            Notes:
                Indicates what system it is.

                
                This attribute is named `systemType` in VSD API.
                
        """
        return self._system_type

    @system_type.setter
    def system_type(self, value):
        """ Set system_type value.

            Notes:
                Indicates what system it is.

                
                This attribute is named `systemType` in VSD API.
                
        """
        self._system_type = value

    

    


# -*- coding: utf-8 -*-
#
# Copyright (c) 2015, Alcatel-Lucent Inc, 2017 Nokia
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the copyright holder nor the names of its contributors
#       may be used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.




from .fetchers import NUPermissionsFetcher


from .fetchers import NUMetadatasFetcher


from .fetchers import NUGlobalMetadatasFetcher

from bambou import NURESTObject


class NUIKEEncryptionprofile(NURESTObject):
    """ Represents a IKEEncryptionprofile in the VSD

        Notes:
            Represents an IKE Profile
    """

    __rest_name__ = "ikeencryptionprofile"
    __resource_name__ = "ikeencryptionprofiles"

    
    ## Constants
    
    CONST_IPSEC_ENCRYPTION_ALGORITHM_AES128 = "AES128"
    
    CONST_ISAKMP_HASH_ALGORITHM_SHA1 = "SHA1"
    
    CONST_ISAKMP_ENCRYPTION_ALGORITHM_AES128 = "AES128"
    
    CONST_ISAKMP_DIFFIE_HELMAN_GROUP_IDENTIFIER_GROUP_2_1024_BIT_DH = "GROUP_2_1024_BIT_DH"
    
    CONST_DPD_MODE_REPLY_ONLY = "REPLY_ONLY"
    
    CONST_ISAKMP_DIFFIE_HELMAN_GROUP_IDENTIFIER_GROUP_15_3072_BIT_DH = "GROUP_15_3072_BIT_DH"
    
    CONST_ENTITY_SCOPE_ENTERPRISE = "ENTERPRISE"
    
    CONST_DPD_MODE_ON_DEMAND = "ON_DEMAND"
    
    CONST_ISAKMP_HASH_ALGORITHM_SHA256 = "SHA256"
    
    CONST_IPSEC_SA_REPLAY_WINDOW_SIZE_WINDOW_SIZE_64 = "WINDOW_SIZE_64"
    
    CONST_IPSEC_SA_REPLAY_WINDOW_SIZE_WINDOW_SIZE_128 = "WINDOW_SIZE_128"
    
    CONST_IPSEC_AUTHENTICATION_ALGORITHM_HMAC_SHA512 = "HMAC_SHA512"
    
    CONST_ENTITY_SCOPE_GLOBAL = "GLOBAL"
    
    CONST_IPSEC_SA_REPLAY_WINDOW_SIZE_WINDOW_SIZE_1024 = "WINDOW_SIZE_1024"
    
    CONST_ISAKMP_DIFFIE_HELMAN_GROUP_IDENTIFIER_GROUP_1_768_BIT_DH = "GROUP_1_768_BIT_DH"
    
    CONST_IPSEC_AUTHENTICATION_ALGORITHM_HMAC_SHA1 = "HMAC_SHA1"
    
    CONST_ISAKMP_AUTHENTICATION_MODE_PRE_SHARED_KEY = "PRE_SHARED_KEY"
    
    CONST_IPSEC_SA_REPLAY_WINDOW_SIZE_WINDOW_SIZE_32 = "WINDOW_SIZE_32"
    
    CONST_IPSEC_ENCRYPTION_ALGORITHM_NULL = "NULL"
    
    CONST_IPSEC_SA_REPLAY_WINDOW_SIZE_WINDOW_SIZE_256 = "WINDOW_SIZE_256"
    
    CONST_ISAKMP_DIFFIE_HELMAN_GROUP_IDENTIFIER_GROUP_14_2048_BIT_DH = "GROUP_14_2048_BIT_DH"
    
    CONST_IPSEC_ENCRYPTION_ALGORITHM_TRIPLE_DES = "TRIPLE_DES"
    
    CONST_IPSEC_AUTHENTICATION_ALGORITHM_HMAC_SHA256 = "HMAC_SHA256"
    
    CONST_ISAKMP_ENCRYPTION_ALGORITHM_AES256 = "AES256"
    
    CONST_IPSEC_AUTHENTICATION_ALGORITHM_HMAC_MD5 = "HMAC_MD5"
    
    CONST_IPSEC_ENCRYPTION_ALGORITHM_AES256 = "AES256"
    
    CONST_IPSEC_ENCRYPTION_ALGORITHM_AES192 = "AES192"
    
    CONST_ISAKMP_ENCRYPTION_ALGORITHM_AES192 = "AES192"
    
    CONST_ISAKMP_ENCRYPTION_ALGORITHM_TRIPLE_DES = "TRIPLE_DES"
    
    CONST_ISAKMP_DIFFIE_HELMAN_GROUP_IDENTIFIER_GROUP_5_1536_BIT_DH = "GROUP_5_1536_BIT_DH"
    
    CONST_IPSEC_SA_REPLAY_WINDOW_SIZE_WINDOW_SIZE_512 = "WINDOW_SIZE_512"
    
    

    def __init__(self, **kwargs):
        """ Initializes a IKEEncryptionprofile instance

            Notes:
                You can specify all parameters while calling this methods.
                A special argument named `data` will enable you to load the
                object from a Python dictionary

            Examples:
                >>> ikeencryptionprofile = NUIKEEncryptionprofile(id=u'xxxx-xxx-xxx-xxx', name=u'IKEEncryptionprofile')
                >>> ikeencryptionprofile = NUIKEEncryptionprofile(data=my_dict)
        """

        super(NUIKEEncryptionprofile, self).__init__()

        # Read/Write Attributes
        
        self._dpd_interval = None
        self._dpd_mode = None
        self._dpd_timeout = None
        self._ipsec_authentication_algorithm = None
        self._ipsec_dont_fragment = None
        self._ipsec_enable_pfs = None
        self._ipsec_encryption_algorithm = None
        self._ipsec_pre_fragment = None
        self._ipsec_sa_lifetime = None
        self._ipsec_sa_replay_window_size = None
        self._ipsec_sa_replay_window_size_value = None
        self._isakmp_authentication_mode = None
        self._isakmp_diffie_helman_group_identifier = None
        self._isakmp_encryption_algorithm = None
        self._isakmp_encryption_key_lifetime = None
        self._isakmp_hash_algorithm = None
        self._name = None
        self._last_updated_by = None
        self._last_updated_date = None
        self._sequence = None
        self._description = None
        self._embedded_metadata = None
        self._entity_scope = None
        self._creation_date = None
        self._associated_enterprise_id = None
        self._owner = None
        self._external_id = None
        
        self.expose_attribute(local_name="dpd_interval", remote_name="DPDInterval", attribute_type=int, is_required=False, is_unique=False)
        self.expose_attribute(local_name="dpd_mode", remote_name="DPDMode", attribute_type=str, is_required=False, is_unique=False, choices=[u'ON_DEMAND', u'REPLY_ONLY'])
        self.expose_attribute(local_name="dpd_timeout", remote_name="DPDTimeout", attribute_type=int, is_required=False, is_unique=False)
        self.expose_attribute(local_name="ipsec_authentication_algorithm", remote_name="IPsecAuthenticationAlgorithm", attribute_type=str, is_required=False, is_unique=False, choices=[u'HMAC_MD5', u'HMAC_SHA1', u'HMAC_SHA256', u'HMAC_SHA512'])
        self.expose_attribute(local_name="ipsec_dont_fragment", remote_name="IPsecDontFragment", attribute_type=bool, is_required=False, is_unique=False)
        self.expose_attribute(local_name="ipsec_enable_pfs", remote_name="IPsecEnablePFS", attribute_type=bool, is_required=False, is_unique=False)
        self.expose_attribute(local_name="ipsec_encryption_algorithm", remote_name="IPsecEncryptionAlgorithm", attribute_type=str, is_required=False, is_unique=False, choices=[u'AES128', u'AES192', u'AES256', u'NULL', u'TRIPLE_DES'])
        self.expose_attribute(local_name="ipsec_pre_fragment", remote_name="IPsecPreFragment", attribute_type=bool, is_required=False, is_unique=False)
        self.expose_attribute(local_name="ipsec_sa_lifetime", remote_name="IPsecSALifetime", attribute_type=int, is_required=False, is_unique=False)
        self.expose_attribute(local_name="ipsec_sa_replay_window_size", remote_name="IPsecSAReplayWindowSize", attribute_type=str, is_required=False, is_unique=False, choices=[u'WINDOW_SIZE_1024', u'WINDOW_SIZE_128', u'WINDOW_SIZE_256', u'WINDOW_SIZE_32', u'WINDOW_SIZE_512', u'WINDOW_SIZE_64'])
        self.expose_attribute(local_name="ipsec_sa_replay_window_size_value", remote_name="IPsecSAReplayWindowSizeValue", attribute_type=int, is_required=False, is_unique=False)
        self.expose_attribute(local_name="isakmp_authentication_mode", remote_name="ISAKMPAuthenticationMode", attribute_type=str, is_required=False, is_unique=False, choices=[u'PRE_SHARED_KEY'])
        self.expose_attribute(local_name="isakmp_diffie_helman_group_identifier", remote_name="ISAKMPDiffieHelmanGroupIdentifier", attribute_type=str, is_required=False, is_unique=False, choices=[u'GROUP_14_2048_BIT_DH', u'GROUP_15_3072_BIT_DH', u'GROUP_1_768_BIT_DH', u'GROUP_2_1024_BIT_DH', u'GROUP_5_1536_BIT_DH'])
        self.expose_attribute(local_name="isakmp_encryption_algorithm", remote_name="ISAKMPEncryptionAlgorithm", attribute_type=str, is_required=False, is_unique=False, choices=[u'AES128', u'AES192', u'AES256', u'TRIPLE_DES'])
        self.expose_attribute(local_name="isakmp_encryption_key_lifetime", remote_name="ISAKMPEncryptionKeyLifetime", attribute_type=int, is_required=False, is_unique=False)
        self.expose_attribute(local_name="isakmp_hash_algorithm", remote_name="ISAKMPHashAlgorithm", attribute_type=str, is_required=False, is_unique=False, choices=[u'SHA1', u'SHA256'])
        self.expose_attribute(local_name="name", remote_name="name", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="last_updated_by", remote_name="lastUpdatedBy", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="last_updated_date", remote_name="lastUpdatedDate", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="sequence", remote_name="sequence", attribute_type=int, is_required=False, is_unique=False)
        self.expose_attribute(local_name="description", remote_name="description", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="embedded_metadata", remote_name="embeddedMetadata", attribute_type=list, is_required=False, is_unique=False)
        self.expose_attribute(local_name="entity_scope", remote_name="entityScope", attribute_type=str, is_required=False, is_unique=False, choices=[u'ENTERPRISE', u'GLOBAL'])
        self.expose_attribute(local_name="creation_date", remote_name="creationDate", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="associated_enterprise_id", remote_name="associatedEnterpriseID", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="owner", remote_name="owner", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="external_id", remote_name="externalID", attribute_type=str, is_required=False, is_unique=True)
        

        # Fetchers
        
        
        self.permissions = NUPermissionsFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.metadatas = NUMetadatasFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.global_metadatas = NUGlobalMetadatasFetcher.fetcher_with_object(parent_object=self, relationship="child")
        

        self._compute_args(**kwargs)

    # Properties
    
    @property
    def dpd_interval(self):
        """ Get dpd_interval value.

            Notes:
                ISAKMP Keep Alive Interval.

                
                This attribute is named `DPDInterval` in VSD API.
                
        """
        return self._dpd_interval

    @dpd_interval.setter
    def dpd_interval(self, value):
        """ Set dpd_interval value.

            Notes:
                ISAKMP Keep Alive Interval.

                
                This attribute is named `DPDInterval` in VSD API.
                
        """
        self._dpd_interval = value

    
    @property
    def dpd_mode(self):
        """ Get dpd_mode value.

            Notes:
                DPD Mode.

                
                This attribute is named `DPDMode` in VSD API.
                
        """
        return self._dpd_mode

    @dpd_mode.setter
    def dpd_mode(self, value):
        """ Set dpd_mode value.

            Notes:
                DPD Mode.

                
                This attribute is named `DPDMode` in VSD API.
                
        """
        self._dpd_mode = value

    
    @property
    def dpd_timeout(self):
        """ Get dpd_timeout value.

            Notes:
                DPD Timeout.

                
                This attribute is named `DPDTimeout` in VSD API.
                
        """
        return self._dpd_timeout

    @dpd_timeout.setter
    def dpd_timeout(self, value):
        """ Set dpd_timeout value.

            Notes:
                DPD Timeout.

                
                This attribute is named `DPDTimeout` in VSD API.
                
        """
        self._dpd_timeout = value

    
    @property
    def ipsec_authentication_algorithm(self):
        """ Get ipsec_authentication_algorithm value.

            Notes:
                IPsec Authentication Algorithm.

                
                This attribute is named `IPsecAuthenticationAlgorithm` in VSD API.
                
        """
        return self._ipsec_authentication_algorithm

    @ipsec_authentication_algorithm.setter
    def ipsec_authentication_algorithm(self, value):
        """ Set ipsec_authentication_algorithm value.

            Notes:
                IPsec Authentication Algorithm.

                
                This attribute is named `IPsecAuthenticationAlgorithm` in VSD API.
                
        """
        self._ipsec_authentication_algorithm = value

    
    @property
    def ipsec_dont_fragment(self):
        """ Get ipsec_dont_fragment value.

            Notes:
                IPsec Don't Fragment

                
                This attribute is named `IPsecDontFragment` in VSD API.
                
        """
        return self._ipsec_dont_fragment

    @ipsec_dont_fragment.setter
    def ipsec_dont_fragment(self, value):
        """ Set ipsec_dont_fragment value.

            Notes:
                IPsec Don't Fragment

                
                This attribute is named `IPsecDontFragment` in VSD API.
                
        """
        self._ipsec_dont_fragment = value

    
    @property
    def ipsec_enable_pfs(self):
        """ Get ipsec_enable_pfs value.

            Notes:
                IPsec Enable PFS

                
                This attribute is named `IPsecEnablePFS` in VSD API.
                
        """
        return self._ipsec_enable_pfs

    @ipsec_enable_pfs.setter
    def ipsec_enable_pfs(self, value):
        """ Set ipsec_enable_pfs value.

            Notes:
                IPsec Enable PFS

                
                This attribute is named `IPsecEnablePFS` in VSD API.
                
        """
        self._ipsec_enable_pfs = value

    
    @property
    def ipsec_encryption_algorithm(self):
        """ Get ipsec_encryption_algorithm value.

            Notes:
                IPsec Encryption Algorithm.

                
                This attribute is named `IPsecEncryptionAlgorithm` in VSD API.
                
        """
        return self._ipsec_encryption_algorithm

    @ipsec_encryption_algorithm.setter
    def ipsec_encryption_algorithm(self, value):
        """ Set ipsec_encryption_algorithm value.

            Notes:
                IPsec Encryption Algorithm.

                
                This attribute is named `IPsecEncryptionAlgorithm` in VSD API.
                
        """
        self._ipsec_encryption_algorithm = value

    
    @property
    def ipsec_pre_fragment(self):
        """ Get ipsec_pre_fragment value.

            Notes:
                IPsec PreFragment

                
                This attribute is named `IPsecPreFragment` in VSD API.
                
        """
        return self._ipsec_pre_fragment

    @ipsec_pre_fragment.setter
    def ipsec_pre_fragment(self, value):
        """ Set ipsec_pre_fragment value.

            Notes:
                IPsec PreFragment

                
                This attribute is named `IPsecPreFragment` in VSD API.
                
        """
        self._ipsec_pre_fragment = value

    
    @property
    def ipsec_sa_lifetime(self):
        """ Get ipsec_sa_lifetime value.

            Notes:
                IPsec SA Lifetime in Seconds.

                
                This attribute is named `IPsecSALifetime` in VSD API.
                
        """
        return self._ipsec_sa_lifetime

    @ipsec_sa_lifetime.setter
    def ipsec_sa_lifetime(self, value):
        """ Set ipsec_sa_lifetime value.

            Notes:
                IPsec SA Lifetime in Seconds.

                
                This attribute is named `IPsecSALifetime` in VSD API.
                
        """
        self._ipsec_sa_lifetime = value

    
    @property
    def ipsec_sa_replay_window_size(self):
        """ Get ipsec_sa_replay_window_size value.

            Notes:
                IPsec Replay Window Size in Packets.

                
                This attribute is named `IPsecSAReplayWindowSize` in VSD API.
                
        """
        return self._ipsec_sa_replay_window_size

    @ipsec_sa_replay_window_size.setter
    def ipsec_sa_replay_window_size(self, value):
        """ Set ipsec_sa_replay_window_size value.

            Notes:
                IPsec Replay Window Size in Packets.

                
                This attribute is named `IPsecSAReplayWindowSize` in VSD API.
                
        """
        self._ipsec_sa_replay_window_size = value

    
    @property
    def ipsec_sa_replay_window_size_value(self):
        """ Get ipsec_sa_replay_window_size_value value.

            Notes:
                IPsec Replay Window Size in Packets.

                
                This attribute is named `IPsecSAReplayWindowSizeValue` in VSD API.
                
        """
        return self._ipsec_sa_replay_window_size_value

    @ipsec_sa_replay_window_size_value.setter
    def ipsec_sa_replay_window_size_value(self, value):
        """ Set ipsec_sa_replay_window_size_value value.

            Notes:
                IPsec Replay Window Size in Packets.

                
                This attribute is named `IPsecSAReplayWindowSizeValue` in VSD API.
                
        """
        self._ipsec_sa_replay_window_size_value = value

    
    @property
    def isakmp_authentication_mode(self):
        """ Get isakmp_authentication_mode value.

            Notes:
                ISAKMP Authentication Algorithm.

                
                This attribute is named `ISAKMPAuthenticationMode` in VSD API.
                
        """
        return self._isakmp_authentication_mode

    @isakmp_authentication_mode.setter
    def isakmp_authentication_mode(self, value):
        """ Set isakmp_authentication_mode value.

            Notes:
                ISAKMP Authentication Algorithm.

                
                This attribute is named `ISAKMPAuthenticationMode` in VSD API.
                
        """
        self._isakmp_authentication_mode = value

    
    @property
    def isakmp_diffie_helman_group_identifier(self):
        """ Get isakmp_diffie_helman_group_identifier value.

            Notes:
                ISAKMP Diffie-Helman Group Identifier.

                
                This attribute is named `ISAKMPDiffieHelmanGroupIdentifier` in VSD API.
                
        """
        return self._isakmp_diffie_helman_group_identifier

    @isakmp_diffie_helman_group_identifier.setter
    def isakmp_diffie_helman_group_identifier(self, value):
        """ Set isakmp_diffie_helman_group_identifier value.

            Notes:
                ISAKMP Diffie-Helman Group Identifier.

                
                This attribute is named `ISAKMPDiffieHelmanGroupIdentifier` in VSD API.
                
        """
        self._isakmp_diffie_helman_group_identifier = value

    
    @property
    def isakmp_encryption_algorithm(self):
        """ Get isakmp_encryption_algorithm value.

            Notes:
                ISAKMP Encryption Algorithm.

                
                This attribute is named `ISAKMPEncryptionAlgorithm` in VSD API.
                
        """
        return self._isakmp_encryption_algorithm

    @isakmp_encryption_algorithm.setter
    def isakmp_encryption_algorithm(self, value):
        """ Set isakmp_encryption_algorithm value.

            Notes:
                ISAKMP Encryption Algorithm.

                
                This attribute is named `ISAKMPEncryptionAlgorithm` in VSD API.
                
        """
        self._isakmp_encryption_algorithm = value

    
    @property
    def isakmp_encryption_key_lifetime(self):
        """ Get isakmp_encryption_key_lifetime value.

            Notes:
                ISAKMP Encryption Key Lifetime in Seconds

                
                This attribute is named `ISAKMPEncryptionKeyLifetime` in VSD API.
                
        """
        return self._isakmp_encryption_key_lifetime

    @isakmp_encryption_key_lifetime.setter
    def isakmp_encryption_key_lifetime(self, value):
        """ Set isakmp_encryption_key_lifetime value.

            Notes:
                ISAKMP Encryption Key Lifetime in Seconds

                
                This attribute is named `ISAKMPEncryptionKeyLifetime` in VSD API.
                
        """
        self._isakmp_encryption_key_lifetime = value

    
    @property
    def isakmp_hash_algorithm(self):
        """ Get isakmp_hash_algorithm value.

            Notes:
                ISAKMP Hash Algorithm.

                
                This attribute is named `ISAKMPHashAlgorithm` in VSD API.
                
        """
        return self._isakmp_hash_algorithm

    @isakmp_hash_algorithm.setter
    def isakmp_hash_algorithm(self, value):
        """ Set isakmp_hash_algorithm value.

            Notes:
                ISAKMP Hash Algorithm.

                
                This attribute is named `ISAKMPHashAlgorithm` in VSD API.
                
        """
        self._isakmp_hash_algorithm = value

    
    @property
    def name(self):
        """ Get name value.

            Notes:
                Name of the Encryption Profile

                
        """
        return self._name

    @name.setter
    def name(self, value):
        """ Set name value.

            Notes:
                Name of the Encryption Profile

                
        """
        self._name = value

    
    @property
    def last_updated_by(self):
        """ Get last_updated_by value.

            Notes:
                ID of the user who last updated the object.

                
                This attribute is named `lastUpdatedBy` in VSD API.
                
        """
        return self._last_updated_by

    @last_updated_by.setter
    def last_updated_by(self, value):
        """ Set last_updated_by value.

            Notes:
                ID of the user who last updated the object.

                
                This attribute is named `lastUpdatedBy` in VSD API.
                
        """
        self._last_updated_by = value

    
    @property
    def last_updated_date(self):
        """ Get last_updated_date value.

            Notes:
                Time stamp when this object was last updated.

                
                This attribute is named `lastUpdatedDate` in VSD API.
                
        """
        return self._last_updated_date

    @last_updated_date.setter
    def last_updated_date(self, value):
        """ Set last_updated_date value.

            Notes:
                Time stamp when this object was last updated.

                
                This attribute is named `lastUpdatedDate` in VSD API.
                
        """
        self._last_updated_date = value

    
    @property
    def sequence(self):
        """ Get sequence value.

            Notes:
                None

                
        """
        return self._sequence

    @sequence.setter
    def sequence(self, value):
        """ Set sequence value.

            Notes:
                None

                
        """
        self._sequence = value

    
    @property
    def description(self):
        """ Get description value.

            Notes:
                A description of the Profile instance created.

                
        """
        return self._description

    @description.setter
    def description(self, value):
        """ Set description value.

            Notes:
                A description of the Profile instance created.

                
        """
        self._description = value

    
    @property
    def embedded_metadata(self):
        """ Get embedded_metadata value.

            Notes:
                Metadata objects associated with this entity. This will contain a list of Metadata objects if the API request is made using the special flag to enable the embedded Metadata feature. Only a maximum of Metadata objects is returned based on the value set in the system configuration.

                
                This attribute is named `embeddedMetadata` in VSD API.
                
        """
        return self._embedded_metadata

    @embedded_metadata.setter
    def embedded_metadata(self, value):
        """ Set embedded_metadata value.

            Notes:
                Metadata objects associated with this entity. This will contain a list of Metadata objects if the API request is made using the special flag to enable the embedded Metadata feature. Only a maximum of Metadata objects is returned based on the value set in the system configuration.

                
                This attribute is named `embeddedMetadata` in VSD API.
                
        """
        self._embedded_metadata = value

    
    @property
    def entity_scope(self):
        """ Get entity_scope value.

            Notes:
                Specify if scope of entity is Data center or Enterprise level

                
                This attribute is named `entityScope` in VSD API.
                
        """
        return self._entity_scope

    @entity_scope.setter
    def entity_scope(self, value):
        """ Set entity_scope value.

            Notes:
                Specify if scope of entity is Data center or Enterprise level

                
                This attribute is named `entityScope` in VSD API.
                
        """
        self._entity_scope = value

    
    @property
    def creation_date(self):
        """ Get creation_date value.

            Notes:
                Time stamp when this object was created.

                
                This attribute is named `creationDate` in VSD API.
                
        """
        return self._creation_date

    @creation_date.setter
    def creation_date(self, value):
        """ Set creation_date value.

            Notes:
                Time stamp when this object was created.

                
                This attribute is named `creationDate` in VSD API.
                
        """
        self._creation_date = value

    
    @property
    def associated_enterprise_id(self):
        """ Get associated_enterprise_id value.

            Notes:
                The ID of the associated Enterprise

                
                This attribute is named `associatedEnterpriseID` in VSD API.
                
        """
        return self._associated_enterprise_id

    @associated_enterprise_id.setter
    def associated_enterprise_id(self, value):
        """ Set associated_enterprise_id value.

            Notes:
                The ID of the associated Enterprise

                
                This attribute is named `associatedEnterpriseID` in VSD API.
                
        """
        self._associated_enterprise_id = value

    
    @property
    def owner(self):
        """ Get owner value.

            Notes:
                Identifies the user that has created this object.

                
        """
        return self._owner

    @owner.setter
    def owner(self, value):
        """ Set owner value.

            Notes:
                Identifies the user that has created this object.

                
        """
        self._owner = value

    
    @property
    def external_id(self):
        """ Get external_id value.

            Notes:
                External object ID. Used for integration with third party systems

                
                This attribute is named `externalID` in VSD API.
                
        """
        return self._external_id

    @external_id.setter
    def external_id(self, value):
        """ Set external_id value.

            Notes:
                External object ID. Used for integration with third party systems

                
                This attribute is named `externalID` in VSD API.
                
        """
        self._external_id = value

    

    


# -*- coding: utf-8 -*-
#
# Copyright (c) 2015, Alcatel-Lucent Inc, 2017 Nokia
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the copyright holder nor the names of its contributors
#       may be used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.



from bambou import NURESTObject


class NUCertificateMetadata(NURESTObject):
    """ Represents a CertificateMetadata in the VSD

        Notes:
            An object to store metadata about an end entity's certificate.
    """

    __rest_name__ = "None"
    __resource_name__ = "None"

    

    def __init__(self, **kwargs):
        """ Initializes a CertificateMetadata instance

            Notes:
                You can specify all parameters while calling this methods.
                A special argument named `data` will enable you to load the
                object from a Python dictionary

            Examples:
                >>> certificatemetadata = NUCertificateMetadata(id=u'xxxx-xxx-xxx-xxx', name=u'CertificateMetadata')
                >>> certificatemetadata = NUCertificateMetadata(data=my_dict)
        """

        super(NUCertificateMetadata, self).__init__()

        # Read/Write Attributes
        
        self._sha1_fingerprint = None
        self._serial = None
        self._not_after = None
        self._not_before = None
        self._issuer = None
        self._subject = None
        
        self.expose_attribute(local_name="sha1_fingerprint", remote_name="SHA1Fingerprint", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="serial", remote_name="serial", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="not_after", remote_name="notAfter", attribute_type=float, is_required=False, is_unique=False)
        self.expose_attribute(local_name="not_before", remote_name="notBefore", attribute_type=float, is_required=False, is_unique=False)
        self.expose_attribute(local_name="issuer", remote_name="issuer", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="subject", remote_name="subject", attribute_type=str, is_required=False, is_unique=False)
        

        self._compute_args(**kwargs)

    # Properties
    
    @property
    def sha1_fingerprint(self):
        """ Get sha1_fingerprint value.

            Notes:
                The SHA1 FingerPrint of the certificate.

                
                This attribute is named `SHA1Fingerprint` in VSD API.
                
        """
        return self._sha1_fingerprint

    @sha1_fingerprint.setter
    def sha1_fingerprint(self, value):
        """ Set sha1_fingerprint value.

            Notes:
                The SHA1 FingerPrint of the certificate.

                
                This attribute is named `SHA1Fingerprint` in VSD API.
                
        """
        self._sha1_fingerprint = value

    
    @property
    def serial(self):
        """ Get serial value.

            Notes:
                The unique serial number of the certificate assigned by the certificate authority.

                
        """
        return self._serial

    @serial.setter
    def serial(self, value):
        """ Set serial value.

            Notes:
                The unique serial number of the certificate assigned by the certificate authority.

                
        """
        self._serial = value

    
    @property
    def not_after(self):
        """ Get not_after value.

            Notes:
                The end date of the certificate validity period.

                
                This attribute is named `notAfter` in VSD API.
                
        """
        return self._not_after

    @not_after.setter
    def not_after(self, value):
        """ Set not_after value.

            Notes:
                The end date of the certificate validity period.

                
                This attribute is named `notAfter` in VSD API.
                
        """
        self._not_after = value

    
    @property
    def not_before(self):
        """ Get not_before value.

            Notes:
                The start date of the certificate validity period.

                
                This attribute is named `notBefore` in VSD API.
                
        """
        return self._not_before

    @not_before.setter
    def not_before(self, value):
        """ Set not_before value.

            Notes:
                The start date of the certificate validity period.

                
                This attribute is named `notBefore` in VSD API.
                
        """
        self._not_before = value

    
    @property
    def issuer(self):
        """ Get issuer value.

            Notes:
                The distinguished name of the certificate issuer.

                
        """
        return self._issuer

    @issuer.setter
    def issuer(self, value):
        """ Set issuer value.

            Notes:
                The distinguished name of the certificate issuer.

                
        """
        self._issuer = value

    
    @property
    def subject(self):
        """ Get subject value.

            Notes:
                The subject distinguished name of the certificate.

                
        """
        return self._subject

    @subject.setter
    def subject(self, value):
        """ Set subject value.

            Notes:
                The subject distinguished name of the certificate.

                
        """
        self._subject = value

    

    


# -*- coding: utf-8 -*-
#
# Copyright (c) 2015, Alcatel-Lucent Inc, 2017 Nokia
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the copyright holder nor the names of its contributors
#       may be used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.




from .fetchers import NUIDPProfileActionsFetcher

from bambou import NURESTObject


class NUIDPProfile(NURESTObject):
    """ Represents a IDPProfile in the VSD

        Notes:
            IDP Profile/Rules are used to detect intrusion attempts and/or inspect network traffic and take appropriate action.
    """

    __rest_name__ = "idpprofile"
    __resource_name__ = "idpprofiles"

    

    def __init__(self, **kwargs):
        """ Initializes a IDPProfile instance

            Notes:
                You can specify all parameters while calling this methods.
                A special argument named `data` will enable you to load the
                object from a Python dictionary

            Examples:
                >>> idpprofile = NUIDPProfile(id=u'xxxx-xxx-xxx-xxx', name=u'IDPProfile')
                >>> idpprofile = NUIDPProfile(data=my_dict)
        """

        super(NUIDPProfile, self).__init__()

        # Read/Write Attributes
        
        self._name = None
        self._description = None
        self._protect_against_insertion_evasion = None
        self._associated_enterprise_id = None
        
        self.expose_attribute(local_name="name", remote_name="name", attribute_type=str, is_required=True, is_unique=False)
        self.expose_attribute(local_name="description", remote_name="description", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="protect_against_insertion_evasion", remote_name="protectAgainstInsertionEvasion", attribute_type=bool, is_required=False, is_unique=False)
        self.expose_attribute(local_name="associated_enterprise_id", remote_name="associatedEnterpriseID", attribute_type=str, is_required=False, is_unique=False)
        

        # Fetchers
        
        
        self.idp_profile_actions = NUIDPProfileActionsFetcher.fetcher_with_object(parent_object=self, relationship="child")
        

        self._compute_args(**kwargs)

    # Properties
    
    @property
    def name(self):
        """ Get name value.

            Notes:
                Symbolic name of the IDP Rule

                
        """
        return self._name

    @name.setter
    def name(self, value):
        """ Set name value.

            Notes:
                Symbolic name of the IDP Rule

                
        """
        self._name = value

    
    @property
    def description(self):
        """ Get description value.

            Notes:
                Descriptive text for IDP Profile

                
        """
        return self._description

    @description.setter
    def description(self, value):
        """ Set description value.

            Notes:
                Descriptive text for IDP Profile

                
        """
        self._description = value

    
    @property
    def protect_against_insertion_evasion(self):
        """ Get protect_against_insertion_evasion value.

            Notes:
                Enable protection against insertion/evasion attacks.

                
                This attribute is named `protectAgainstInsertionEvasion` in VSD API.
                
        """
        return self._protect_against_insertion_evasion

    @protect_against_insertion_evasion.setter
    def protect_against_insertion_evasion(self, value):
        """ Set protect_against_insertion_evasion value.

            Notes:
                Enable protection against insertion/evasion attacks.

                
                This attribute is named `protectAgainstInsertionEvasion` in VSD API.
                
        """
        self._protect_against_insertion_evasion = value

    
    @property
    def associated_enterprise_id(self):
        """ Get associated_enterprise_id value.

            Notes:
                The ID of the associated Enterprise

                
                This attribute is named `associatedEnterpriseID` in VSD API.
                
        """
        return self._associated_enterprise_id

    @associated_enterprise_id.setter
    def associated_enterprise_id(self, value):
        """ Set associated_enterprise_id value.

            Notes:
                The ID of the associated Enterprise

                
                This attribute is named `associatedEnterpriseID` in VSD API.
                
        """
        self._associated_enterprise_id = value

    

    


# -*- coding: utf-8 -*-
#
# Copyright (c) 2015, Alcatel-Lucent Inc, 2017 Nokia
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the copyright holder nor the names of its contributors
#       may be used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.




from .fetchers import NUPermissionsFetcher


from .fetchers import NUMetadatasFetcher


from .fetchers import NUGlobalMetadatasFetcher


from .fetchers import NUVNFInterfaceDescriptorsFetcher

from bambou import NURESTObject


class NUVNFDescriptor(NURESTObject):
    """ Represents a VNFDescriptor in the VSD

        Notes:
            The behavioral and deployment information of a VNF is defined in the VNF descriptor template. The template is based on the libvirt domain XML and is on-boarded in a VNF catalog. The resource requirements for CPU, memory and storage are defined in this screen and the rest of the template is inherited from the VNF Metadata object.
    """

    __rest_name__ = "vnfdescriptor"
    __resource_name__ = "vnfdescriptors"

    
    ## Constants
    
    CONST_TYPE_WAN_OPT = "WAN_OPT"
    
    CONST_ENTITY_SCOPE_GLOBAL = "GLOBAL"
    
    CONST_TYPE_FIREWALL = "FIREWALL"
    
    CONST_TYPE_THREAT_PREVENTION = "THREAT_PREVENTION"
    
    CONST_ENTITY_SCOPE_ENTERPRISE = "ENTERPRISE"
    
    

    def __init__(self, **kwargs):
        """ Initializes a VNFDescriptor instance

            Notes:
                You can specify all parameters while calling this methods.
                A special argument named `data` will enable you to load the
                object from a Python dictionary

            Examples:
                >>> vnfdescriptor = NUVNFDescriptor(id=u'xxxx-xxx-xxx-xxx', name=u'VNFDescriptor')
                >>> vnfdescriptor = NUVNFDescriptor(data=my_dict)
        """

        super(NUVNFDescriptor, self).__init__()

        # Read/Write Attributes
        
        self._cpu_count = None
        self._name = None
        self._memory_mb = None
        self._vendor = None
        self._description = None
        self._metadata_id = None
        self._visible = None
        self._embedded_metadata = None
        self._entity_scope = None
        self._associated_vnf_threshold_policy_id = None
        self._storage_gb = None
        self._external_id = None
        self._type = None
        
        self.expose_attribute(local_name="cpu_count", remote_name="CPUCount", attribute_type=int, is_required=True, is_unique=False)
        self.expose_attribute(local_name="name", remote_name="name", attribute_type=str, is_required=True, is_unique=False)
        self.expose_attribute(local_name="memory_mb", remote_name="memoryMB", attribute_type=int, is_required=True, is_unique=False)
        self.expose_attribute(local_name="vendor", remote_name="vendor", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="description", remote_name="description", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="metadata_id", remote_name="metadataID", attribute_type=str, is_required=True, is_unique=False)
        self.expose_attribute(local_name="visible", remote_name="visible", attribute_type=bool, is_required=False, is_unique=False)
        self.expose_attribute(local_name="embedded_metadata", remote_name="embeddedMetadata", attribute_type=list, is_required=False, is_unique=False)
        self.expose_attribute(local_name="entity_scope", remote_name="entityScope", attribute_type=str, is_required=False, is_unique=False, choices=[u'ENTERPRISE', u'GLOBAL'])
        self.expose_attribute(local_name="associated_vnf_threshold_policy_id", remote_name="associatedVNFThresholdPolicyID", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="storage_gb", remote_name="storageGB", attribute_type=int, is_required=True, is_unique=False)
        self.expose_attribute(local_name="external_id", remote_name="externalID", attribute_type=str, is_required=False, is_unique=True)
        self.expose_attribute(local_name="type", remote_name="type", attribute_type=str, is_required=False, is_unique=False, choices=[u'FIREWALL', u'THREAT_PREVENTION', u'WAN_OPT'])
        

        # Fetchers
        
        
        self.permissions = NUPermissionsFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.metadatas = NUMetadatasFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.global_metadatas = NUGlobalMetadatasFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.vnf_interface_descriptors = NUVNFInterfaceDescriptorsFetcher.fetcher_with_object(parent_object=self, relationship="child")
        

        self._compute_args(**kwargs)

    # Properties
    
    @property
    def cpu_count(self):
        """ Get cpu_count value.

            Notes:
                Number of CPUs to be allocated VNF instance when deployed

                
                This attribute is named `CPUCount` in VSD API.
                
        """
        return self._cpu_count

    @cpu_count.setter
    def cpu_count(self, value):
        """ Set cpu_count value.

            Notes:
                Number of CPUs to be allocated VNF instance when deployed

                
                This attribute is named `CPUCount` in VSD API.
                
        """
        self._cpu_count = value

    
    @property
    def name(self):
        """ Get name value.

            Notes:
                Name of the VNF Descriptor

                
        """
        return self._name

    @name.setter
    def name(self, value):
        """ Set name value.

            Notes:
                Name of the VNF Descriptor

                
        """
        self._name = value

    
    @property
    def memory_mb(self):
        """ Get memory_mb value.

            Notes:
                Memory (in MB) to be allocated for VNF instance when deployed

                
                This attribute is named `memoryMB` in VSD API.
                
        """
        return self._memory_mb

    @memory_mb.setter
    def memory_mb(self, value):
        """ Set memory_mb value.

            Notes:
                Memory (in MB) to be allocated for VNF instance when deployed

                
                This attribute is named `memoryMB` in VSD API.
                
        """
        self._memory_mb = value

    
    @property
    def vendor(self):
        """ Get vendor value.

            Notes:
                The vendor generating this VNF Descriptor

                
        """
        return self._vendor

    @vendor.setter
    def vendor(self, value):
        """ Set vendor value.

            Notes:
                The vendor generating this VNF Descriptor

                
        """
        self._vendor = value

    
    @property
    def description(self):
        """ Get description value.

            Notes:
                A description of the VNF Descriptor

                
        """
        return self._description

    @description.setter
    def description(self, value):
        """ Set description value.

            Notes:
                A description of the VNF Descriptor

                
        """
        self._description = value

    
    @property
    def metadata_id(self):
        """ Get metadata_id value.

            Notes:
                Id of referenced Metadata Object

                
                This attribute is named `metadataID` in VSD API.
                
        """
        return self._metadata_id

    @metadata_id.setter
    def metadata_id(self, value):
        """ Set metadata_id value.

            Notes:
                Id of referenced Metadata Object

                
                This attribute is named `metadataID` in VSD API.
                
        """
        self._metadata_id = value

    
    @property
    def visible(self):
        """ Get visible value.

            Notes:
                Controls if descriptor visible in catalog to create new VNF

                
        """
        return self._visible

    @visible.setter
    def visible(self, value):
        """ Set visible value.

            Notes:
                Controls if descriptor visible in catalog to create new VNF

                
        """
        self._visible = value

    
    @property
    def embedded_metadata(self):
        """ Get embedded_metadata value.

            Notes:
                Metadata objects associated with this entity. This will contain a list of Metadata objects if the API request is made using the special flag to enable the embedded Metadata feature. Only a maximum of Metadata objects is returned based on the value set in the system configuration.

                
                This attribute is named `embeddedMetadata` in VSD API.
                
        """
        return self._embedded_metadata

    @embedded_metadata.setter
    def embedded_metadata(self, value):
        """ Set embedded_metadata value.

            Notes:
                Metadata objects associated with this entity. This will contain a list of Metadata objects if the API request is made using the special flag to enable the embedded Metadata feature. Only a maximum of Metadata objects is returned based on the value set in the system configuration.

                
                This attribute is named `embeddedMetadata` in VSD API.
                
        """
        self._embedded_metadata = value

    
    @property
    def entity_scope(self):
        """ Get entity_scope value.

            Notes:
                Specify if scope of entity is Data center or Enterprise level

                
                This attribute is named `entityScope` in VSD API.
                
        """
        return self._entity_scope

    @entity_scope.setter
    def entity_scope(self, value):
        """ Set entity_scope value.

            Notes:
                Specify if scope of entity is Data center or Enterprise level

                
                This attribute is named `entityScope` in VSD API.
                
        """
        self._entity_scope = value

    
    @property
    def associated_vnf_threshold_policy_id(self):
        """ Get associated_vnf_threshold_policy_id value.

            Notes:
                The Id of referenced VNF threshold policy

                
                This attribute is named `associatedVNFThresholdPolicyID` in VSD API.
                
        """
        return self._associated_vnf_threshold_policy_id

    @associated_vnf_threshold_policy_id.setter
    def associated_vnf_threshold_policy_id(self, value):
        """ Set associated_vnf_threshold_policy_id value.

            Notes:
                The Id of referenced VNF threshold policy

                
                This attribute is named `associatedVNFThresholdPolicyID` in VSD API.
                
        """
        self._associated_vnf_threshold_policy_id = value

    
    @property
    def storage_gb(self):
        """ Get storage_gb value.

            Notes:
                Disk storage (in GB) to be allocated VNF instance when deployed

                
                This attribute is named `storageGB` in VSD API.
                
        """
        return self._storage_gb

    @storage_gb.setter
    def storage_gb(self, value):
        """ Set storage_gb value.

            Notes:
                Disk storage (in GB) to be allocated VNF instance when deployed

                
                This attribute is named `storageGB` in VSD API.
                
        """
        self._storage_gb = value

    
    @property
    def external_id(self):
        """ Get external_id value.

            Notes:
                External object ID. Used for integration with third party systems

                
                This attribute is named `externalID` in VSD API.
                
        """
        return self._external_id

    @external_id.setter
    def external_id(self, value):
        """ Set external_id value.

            Notes:
                External object ID. Used for integration with third party systems

                
                This attribute is named `externalID` in VSD API.
                
        """
        self._external_id = value

    
    @property
    def type(self):
        """ Get type value.

            Notes:
                Type of virtual network function

                
        """
        return self._type

    @type.setter
    def type(self, value):
        """ Set type value.

            Notes:
                Type of virtual network function

                
        """
        self._type = value

    

    